#!/usr/bin/env python
"""
Operators present in the FE discretization of (adjoint) Navier-Stokes terms.
"""
from __future__ import print_function
from __future__ import absolute_import
import sympy as s
from six.moves import range

def create_scalar(name, n_ep):
    vec = s.zeros(n_ep, 1)
    for ip in range(n_ep):
        vec[ip,0] = '%s%d' % (name, ip)
    return vec

def create_vector(name, n_ep, dim):
    """ordering is DOF-by-DOF"""
    vec = s.zeros(dim * n_ep, 1)
    for ii in range(dim):
        for ip in range(n_ep):
            vec[n_ep*ii+ip,0] = '%s%d%d' % (name, ii, ip)
    return vec

def create_scalar_base(name, n_ep):
    phi = s.zeros(1, n_ep)
    for ip in range(n_ep):
        phi[0,ip] = '%s%d' % (name, ip)
    return phi

def create_vector_base(name, phic, dim):
    n_ep = phic.shape[1]
    phi = s.zeros(dim, dim * n_ep)
    indx = []
    for ii in range(dim):
        phi[ii,n_ep*ii:n_ep*(ii+1)] = phic
        indx.append(ii)
    return phi, indx

def create_scalar_base_grad(name, phic, dim):
    n_ep = phic.shape[1]
    gc = s.zeros(dim, n_ep)
    for ii in range(dim):
        for ip in range(n_ep):
            gc[ii,ip] = '%s%d%d' % (name, ii, ip)
    return gc

def create_vector_base_grad(name, gc, transpose=False):
    dim, n_ep = gc.shape
    g = s.zeros(dim * dim, dim * n_ep)
    indx = []
    if transpose:
        for ir in range(dim):
            for ic in range(dim):
                g[dim*ir+ic,n_ep*ic:n_ep*(ic+1)] = gc[ir,:]
                indx.append((ic, ir))
    else:
        for ir in range(dim):
            for ic in range(dim):
                g[dim*ir+ic,n_ep*ir:n_ep*(ir+1)] = gc[ic,:]
                indx.append((ir, ic))
    return g, indx

def create_u_operator(u, transpose=False):
    dim = u.shape[0]
    op_u = s.zeros(dim * dim, dim)
    if transpose:
        for ir in range(dim):
            for ic in range(dim):
                op_u[dim*ir+ic,ic] = u[ir]
    else:
        for ii in range(dim):
            op_u[dim*ii:dim*(ii+1),ii] = u
    return op_u

def grad_vector_to_matrix(name, gv):
    dim2 = gv.shape[0]
    dim = int(s.sqrt(dim2))
    gm = s.zeros(dim, dim)
    for ir in range(dim):
        for ic in range(dim):
            gm[ir,ic] = gv[dim*ir+ic,0]
    return gm

def substitute_continuous(expr, names, u, phi):
    pu = phi * u
    for ii in range(phi.rows):
        expr = expr.subs(pu[ii,0], names[ii])
    return expr

def create_vector_var_data(name, phi, vindx, g, gt, vgindx, u):
    gu = g * u
    gum = grad_vector_to_matrix('gum', gu)
    print('g %s:\n' % name, gum)

    gut = gt * u
    gutm = grad_vector_to_matrix('gutm', gut)
    print('gt %s:\n' % name, gutm)

    pu = phi * u
    names = ['c%s%d' % (name, indx) for indx in vindx]
    cu = substitute_continuous(pu, names, u, phi)
    print('continuous %s:\n' % name, cu)

    gnames = ['cg%s%d_%d' % (name, indx[0], indx[1]) for indx in vgindx]
    cgu = substitute_continuous(gu, gnames, u, g)
    cgum = grad_vector_to_matrix('gum', cgu)
    print('continuous g %s:\n' % name, cgum)

    cgut = substitute_continuous(gut, gnames, u, g)
    cgutm = grad_vector_to_matrix('gutm', cgut)
    print('continuous gt %s:\n' % name, cgutm)

    op_u = create_u_operator(cu)
    print(op_u)

    op_ut = create_u_operator(cu, transpose=True)
    print(op_ut)

    out = {
        'g' : gu,
        'g_m' : gum,
        'q' : pu,
        'c' : cu,
        'cg' : cgu,
        'cg_m' : cgum,
        'cgt' : cgut,
        'cgt_m' : cgutm,
        'op' : op_u,
        'opt' : op_ut,
        'names' : names,
        'gnames' : gnames,
    }

    return out

def create_scalar_var_data(name, phi, g, u):
    gu = g * u

    pu = phi * u
    names = ['c%s' % name]
    cu = substitute_continuous(pu, names, u, phi)
    print('continuous %s:\n' % name, cu)

    gnames = ['cg%s_%d' % (name, ii) for ii in range(g.shape[0])]
    cgu = substitute_continuous(gu, gnames, u, g)
    print('continuous g %s:\n' % name, cgu)

    op_gu = create_u_operator(cgu)
    print(op_gu)

    out = {
        'g' : gu,
        'q' : pu,
        'c' : cu,
        'cg' : cgu,
        'gop' : op_gu,
        'names' : names,
        'gnames' : gnames,
    }

    return out

def main():
    n_ep = 3
    dim = 2

    u = create_vector('u', n_ep, dim)
    v = create_vector('v', n_ep, dim)
    b = create_vector('b', n_ep, dim)
    p = create_scalar('p', n_ep)
    q = create_scalar('q', n_ep)
    r = create_scalar('r', n_ep)

    print(u)
    print(v)

    phic = create_scalar_base('phic', n_ep)
    phi, vindx = create_vector_base('phi', phic, dim)
    gc = create_scalar_base_grad('gc', phic, dim)
    g, vgindx = create_vector_base_grad('g', gc)
    gt, aux = create_vector_base_grad('gt', gc, transpose=True)

    print(phi)
    print(phic)
    print(gc)
    print(g)
    print(gt)

    ud = create_vector_var_data('u', phi, vindx, g, gt, vgindx, u)
    vd = create_vector_var_data('v', phi, vindx, g, gt, vgindx, v)
    bd = create_vector_var_data('b', phi, vindx, g, gt, vgindx, b)
    pd = create_scalar_var_data('p', phic, gc, p)
    qd = create_scalar_var_data('q', phic, gc, q)
    rd = create_scalar_var_data('r', phic, gc, r)
    print(list(ud.keys()))

    assert bool(bd['op'].T * g == bd['opt'].T * gt)
    assert bool(bd['opt'].T * g == bd['op'].T * gt)
    assert bool(bd['cgt_m'] == bd['cg_m'].T)

    print('((b * grad) u), v)')
    form1 = vd['c'].T * bd['op'].T * ud['cg']
    form2 = vd['c'].T * bd['opt'].T * ud['cgt']
    print(form1)
    print(form2)
    print(bool(form1 == form2))

    print('((v * grad) u), b)')
    form1 = vd['c'].T * bd['op'].T * ud['cgt']
    form2 = vd['c'].T * bd['opt'].T * ud['cg']
    print(form1)
    print(form2)
    print(bool(form1 == form2))

    print('((u * grad) v), b)')
    form1 = vd['cgt'].T * bd['op'] * ud['c']
    form2 = vd['cg'].T * bd['opt'] * ud['c']
    print(form1)
    print(form2)
    print(bool(form1 == form2))

    print('((b * grad) v), u)')
    form1 = vd['cg'].T * bd['op'] * ud['c']
    form2 = vd['cgt'].T * bd['opt'] * ud['c']
    print(form1)
    print(form2)
    print(bool(form1 == form2))

    print('((v * grad) b), u)')
    form1 = vd['c'].T * bd['cgt_m'] * ud['c']
    form2 = vd['c'].T * bd['cg_m'].T * ud['c']
    print(form1)
    print(form2)
    print(bool(form1 == form2))

    print('((b * grad) u), (b * grad) v)')
    form1 = vd['cg'].T * bd['op'] * bd['op'].T * ud['cg']
    print(form1)

    print('((u * grad) b), (b * grad) v)')
    form1 = vd['cg'].T * bd['op'] * bd['cg_m'] * ud['c']
    print(form1)

    print('(grad p, (b * grad) v)')
    form1 = vd['cg'].T * bd['op'] * pd['cg']
    print(form1)

    print('(grad q, (b * grad) u)')
    form1 = qd['cg'].T * bd['op'].T * ud['cg']
    print(form1)

    print('(grad q, (u * grad) b)')
    form1 = qd['cg'].T * bd['cg_m'] * ud['c']
    print(form1)

    print('(grad r, (u * grad) v)')
    form1 = vd['cgt'].T * rd['gop'] * ud['c']
    print(form1)

    return ud, vd, bd, pd, qd, rd

if __name__ == '__main__':
    ud, vd, bd, pd, qd, rd = main()



r"""
Linear elasticity with a given prestress in one subdomain and a (pre)strain
fibre reinforcement in the other.

Find :math:`\ul{u}` such that:

.. math::
    \int_{\Omega} D_{ijkl}\ e_{ij}(\ul{v}) e_{kl}(\ul{u})
    + \int_{\Omega_1} \sigma_{ij} e_{ij}(\ul{v})
    + \int_{\Omega_2} D^f_{ijkl} e_{ij}(\ul{v}) \left(d_k d_l\right)
    = 0
    \;, \quad \forall \ul{v} \;,

where

.. math::
    D_{ijkl} = \mu (\delta_{ik} \delta_{jl}+\delta_{il} \delta_{jk}) +
    \lambda \ \delta_{ij} \delta_{kl}
    \;.

The stiffness of fibres :math:`D^f_{ijkl}` is defined analogously,
:math:`\ul{d}` is the unit fibre direction vector and :math:`\sigma_{ij}` is
the prestress.

Visualization
-------------

Use the following to see the deformed structure with 10x magnified
displacements::

    $ ./postproc.py block.vtk -b --vector-mode=warp_norm -s 1 --wireframe
"""
from __future__ import absolute_import
import numpy as nm
from sfepy.mechanics.matcoefs import stiffness_from_lame
from sfepy import data_dir

filename_mesh = data_dir + '/meshes/3d/block.mesh'

regions = {
    'Omega' : 'all',
    'Left' : ('vertices in (x < -4.99)', 'facet'),
    'Omega1' : 'vertices in (x < 0.001)',
    'Omega2' : 'vertices in (x > -0.001)',
}

materials = {
    'solid' : ({
        'D' : stiffness_from_lame(3, lam=1e2, mu=1e1),
        'prestress' : 0.1 * nm.array([[1.0], [1.0], [1.0],
                                      [0.5], [0.5], [0.5]],
                                     dtype=nm.float64),
        'DF' : stiffness_from_lame(3, lam=8e0, mu=8e-1),
        'nu' : nm.array([[-0.5], [0.0], [0.5]], dtype=nm.float64),
    },),
}

fields = {
    'displacement': ('real', 'vector', 'Omega', 1),
}

variables = {
    'u' : ('unknown field', 'displacement', 0),
    'v' : ('test field', 'displacement', 'u'),
}

ebcs = {
    'Fixed' : ('Left', {'u.all' : 0.0}),
}

equations = {
    'balance_of_forces' :
    """dw_lin_elastic.2.Omega( solid.D, v, u )
     + dw_lin_prestress.2.Omega1( solid.prestress, v )
     + dw_lin_strain_fib.2.Omega2( solid.DF, solid.nu, v )
     = 0""",
}

solvers = {
    'ls' : ('ls.scipy_direct', {}),
    'newton' : ('nls.newton', {
        'i_max' : 1,
        'eps_a' : 1e-10,
    }),
}



from __future__ import absolute_import
import os
import numpy as nm

from sfepy.base.testing import TestCommon
from six.moves import range

class Test(TestCommon):

    @staticmethod
    def from_conf(conf, options):
        test = Test(conf=conf, options=options)
        test.join = lambda x: os.path.join(test.options.out_dir, x)
        return test

    def test_linearization(self):
        from sfepy.base.base import Struct
        from sfepy.discrete.fem import Mesh, FEDomain, Field
        from sfepy import data_dir

        geometries = ['2_3', '2_4', '3_4', '3_8']
        approx_orders = [1, 2]
        funs = [nm.cos, nm.sin, lambda x: x]

        ok = True
        for geometry in geometries:
            name = os.path.join(data_dir,
                                'meshes/elements/%s_1.mesh' % geometry)
            mesh = Mesh.from_file(name)

            domain = FEDomain('', mesh)
            domain = domain.refine()

            domain.mesh.write(self.join('linearizer-%s-0.mesh' % geometry))

            omega = domain.create_region('Omega', 'all')

            for approx_order in approx_orders:
                for dpn in [1, mesh.dim]:
                    self.report('geometry: %s, approx. order: %d, dpn: %d' %
                                (geometry, approx_order, dpn))

                    field = Field.from_args('fu', nm.float64, dpn, omega,
                                            approx_order=approx_order)

                    cc = field.get_coor()
                    dofs = nm.zeros((field.n_nod, dpn), dtype=nm.float64)

                    for ic in range(dpn):
                        dofs[:, ic] = funs[ic](3 * (cc[:, 0] * cc[:, 1]))

                    vmesh, vdofs, level = field.linearize(dofs,
                                                          min_level=0,
                                                          max_level=3,
                                                          eps=1e-2)

                    if approx_order == 1:
                        _ok = level == 0

                    else:
                        _ok = level > 0
                    self.report('max. refinement level: %d: %s' % (level, _ok))

                    ok = ok and _ok

                    rdofs = nm.zeros((vmesh.n_nod, dpn), dtype=nm.float64)
                    cc = vmesh.coors
                    for ic in range(dpn):
                        rdofs[:, ic] = funs[ic](3 * (cc[:, 0] * cc[:, 1]))

                    _ok = nm.allclose(rdofs, vdofs, rtol=0.0, atol=0.03)
                    self.report('interpolation: %s' % _ok)
                    ok = ok and _ok

                    out = {
                        'u' : Struct(name='output_data',
                                     mode='vertex', data=vdofs,
                                     var_name='u', dofs=None)
                    }

                    name = self.join('linearizer-%s-%d-%d'
                                     % (geometry, approx_order, dpn))

                    vmesh.write(name + '.mesh')
                    vmesh.write(name + '.vtk', out=out)

        return ok



from __future__ import print_function
from __future__ import absolute_import
import sys
from six.moves import range
sys.path.append('.')

import numpy as nm
from sfepy.base.base import Struct

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits.mplot3d.art3d import Poly3DCollection

nm_f64_eps = nm.finfo(nm.float64).eps

def to_ndarray(a):
    if a is None:
        return None
    else:
        a = nm.asarray(a)
        if len(a.shape) == 0:
            a = a.reshape(1)
        return a

class BSpline(Struct):
    """
    B-spline curve representation
    """

    def __init__(self, degree=3, is_cyclic=False, ncp=0):
        """
        Initialize B-spline class.

        Parameters
        ----------
        degree : int
            The degree of the spline function.
        is_cyclic : bool
            Cyclic spline?.
        ncp : int
            The number of control points.
        """
        self.degree = degree
        self.knot_type = None
        self.is_cyclic = is_cyclic
        self.ncp = ncp
        self.knots = None
        self.basis = None
        self.curve_coors = None
        self.cp_coors = None
        self.approx_coors = None
        self.t = None

    def set_control_points(self, coors, cyclic_form=False):
        """
        Set the B-spline control points.

        Parameters
        ----------
        coors : array
            The coordinates of unique control points.
        cyclic_form : bool
            Are the control points in the cyclic form?
        """
        coors = to_ndarray(coors)

        if self.is_cyclic and not cyclic_form:
            coors = nm.vstack((coors, coors[0:self.degree,:]))

        self.cp_coors = coors
        self.ncp = coors.shape[0]

    def get_control_points(self):
        """
        Get the B-spline control points.

        Returns
        -------
        coors : array
            The coordinates of control points.
        """
        if self.is_cyclic:
            return self.cp_coors[:-self.degree,:]
        else:
            return self.cp_coors

    def set_param(self, t):
        """
        Set the B-spline parametric vector.

        Parameters
        ----------
        t : array
            The parameter vector of the B-spline.
        """
        self.t = to_ndarray(t)

        if self.knots is not None:
            endval = self.knots[-(self.degree + 1)]
            idxs = nm.where(self.t == endval)[0]
            self.t[idxs] -= nm_f64_eps

    def set_param_n(self, n=100, knot_range=(0.0, 1.0)):
        """
        Generate the B-spline parametric vector using the number of steps.

        Parameters
        ----------
        n : array
            The number of steps in the B-spline parametric vector.
        """
        self.t = nm.linspace(knot_range[0], knot_range[1], n)
        self.t[-1] -= nm_f64_eps

    @staticmethod
    def basis_function_dg0(t, knots, n):
        """
        Basis function: degree = 0

        Parameters
        ----------
        t : array
            The parametric vector.
        knots : array
            The knot vector.
        n : int
            The number of intervals.

        Returns
        -------
        bfun : array
           The spline basis function evaluated for given values.
        """
        nt = len(t)
        bfun = nm.zeros((nt,n), dtype=nm.float64)
        for ii in range(n):
            idxs = nm.where(nm.logical_and(knots[ii] <= t,
                                           t < knots[ii + 1]))[0]
            bfun[idxs,ii] = 1.0

        return bfun

    @staticmethod
    def basis_function_dg(degree, t, knots, n):
        """
        B-spline basis functions.

        Parameters
        ----------
        degree : int
            The degree of the spline function.
        t : array
            The parametric vector.
        knots : array
            The knot vector.
        n : int
            The number of intervals.

        Returns
        -------
        bfun : array
           The spline basis function evaluated for given values.
        """
        if degree >= 1:
            bfun_dgm1 = BSpline.basis_function_dg(degree - 1, t,
                                                  knots, n + 1)

            nt = len(t)
            bfun = nm.zeros((nt,n), dtype=nm.float64)
            for ii in range(n):
                c1 = t - knots[ii]
                c2 = knots[ii + degree] - knots[ii]

                if nm.abs(c2) > nm_f64_eps:
                     bfun[:,ii] = c1 / c2 * bfun_dgm1[:,ii]

                c1 = knots[ii + degree + 1] - t
                c2 = knots[ii + degree + 1] - knots[ii + 1]
                if nm.abs(c2) > nm_f64_eps:
                    bfun[:,ii] += c1 / c2 * bfun_dgm1[:,ii + 1]

        else:
            bfun = BSpline.basis_function_dg0(t, knots, n)

        return bfun

    def make_knot_vector(self, knot_type='clamped', knot_data=None,
                         knot_range=(0.0, 1.0)):
        """
        Create a knot vector of the requested type.

        Parameters
        ----------
        knot_type : str
            The knot vector type: clamped/cyclic/userdef.
        knot_data :
            The extra knot data.
        """
        if self.is_cyclic and 'cyclic' not in knot_type:
            knot_type = 'cyclic'

        ncp = self.ncp
        dg = self.degree
        n_knots = dg + ncp + 1
        n_inter = n_knots - 2 * dg
        aux = nm.linspace(knot_range[0], knot_range[1], n_inter)
        if knot_type == '' or knot_type == 'cyclic':
            dd = aux[1]
            self.knots = nm.hstack((nm.arange(-dg, 0) * dd,
                                    aux,
                                    nm.arange(1, dg + 1) * dd + 1))

        elif knot_type == 'clamped':
            self.knots = nm.array([aux[0]]* dg + list(aux) + [aux[-1]]* dg,
                                  dtype=nm.float64)

        else:
            raise NotImplementedError

        self.knot_type = knot_type

    def set_knot_vector(self, knots):
        """
        Set the knot vector.

        Parameters
        ----------
        knots : array
            The knot vector.
        """
        self.knot_type = 'userdef'
        self.knots = to_ndarray(knots)

    def get_knot_vector(self):
        """
        Return the knot vector.

        Returns
        -------
        knots : array
            The knot vector.
        """
        return self.knots

    def insert_knot(self, new):
        """
        Insert a new knot into the knot vector.

        Parameters
        ----------
        new : float
            The new knot value.
        """
        kn = self.knots
        dg = self.degree
        ncp = self.ncp
        cp = self.cp_coors
        idx = nm.where(nm.logical_and(kn[:-1] <= new, new < kn[1:]))[0]

        if len(idx) > 0:
            multi = len(nm.where(kn == new)[0])
            if multi < dg:
                # new knot
                newkn = nm.zeros((len(kn) + 1,), dtype=nm.float64)
                newkn[:(idx + 1)] = kn[:(idx + 1)]
                newkn[idx + 1] = new
                newkn[(idx + 2):] = kn[(idx + 1):]
                u1 = idx - dg + 1
                u2 = idx + 1

                # new control points
                newcp = nm.zeros((ncp + 1, cp.shape[1]), dtype=nm.float64)
                newcp[:u1,:] = cp[:u1,:]
                newcp[u2:,:] = cp[(u2 - 1):,:]

                for ii in range(u1, u2):
                    kn1 = kn[ii]
                    kn2 = kn[ii + dg]
                    dd = kn2 - kn1
                    newcp[ii,:] = (kn2 - new) / dd * cp[ii - 1] + \
                                  (new - kn1) / dd * cp[ii]

                self.knots = newkn
                self.cp_coors = newcp
                self.ncp = newcp.shape[0]

                # evaluate B-spline base functions for new configuration
                self.eval_basis()

            else:
                print('knot insertion failed: multiplicity = spline degree!')

        else:
            print('knot insertion failed: out of bounds!')

    def eval_basis(self, t=None, return_val=False):
        """
        Evaluate the basis of the bpsline.

        Parameters
        ----------
        t : array
            The parameter vector of the B-spline.
        """
        if t is not None:
            self.set_param(t)

        if self.knots is None:
            self.make_knot_vector()

        if self.t is None:
            self.set_param_n()

        self.basis = self.basis_function_dg(self.degree, self.t,
                                            self.knots, self.ncp)

        if return_val:
            return self.basis

    def eval(self, t=None, cp_coors=None):
        """
        Evaluate the coordinates of the bpsline curve.

        Parameters
        ----------
        t : array
            The parameter vector of the B-spline.
        cp_coors : array
            The coordinates of the control points.
        """
        if cp_coors is not None:
            self.set_control_points(cp_coors)

        self.eval_basis(t)
        self.curve_coors = nm.dot(self.basis, self.cp_coors)

        return self.curve_coors

    def draw(self, ret_ax=False, ax=None, color='r', cp_id=True):
        """
        Draw B-spline curve.

        Parameters
        ----------
        ret_ax : bool
            Return an axes object?
        ax : axes object
            The axes to which will be drawn.
        color : str
            Line color.
        cp_id : bool
            If True, label control points.
        """
        if self.curve_coors is None:
            self.eval()

        cc = self.curve_coors
        cp = self.cp_coors
        ci = self.approx_coors
        if ci is not None and self.is_cyclic:
            ci = nm.vstack((ci, ci[0,:]))

        if cc.shape[1] == 3:
            if ax is None:
                fig = plt.figure()
                ax = Axes3D(fig)
            ax.plot(cc[:,0], cc[:,1], cc[:,2], color + '-')
            if cp_id:
                ax.plot(cp[:,0], cp[:,1], cp[:,2], 'ko:', alpha=0.6)
            if ci is not None:
                ax.plot(ci[:,0], ci[:,1], ci[:,2], 'b--', alpha=0.6)

        else:
            if ax is None:
                fig = plt.figure()
                ax = fig.add_subplot(111)
            ax.plot(cc[:,0], cc[:,1], color + '-')
            if cp_id:
                ax.plot(cp[:,0], cp[:,1], 'ko:', alpha=0.6)
                for jj, icp in enumerate(self.cp_coors):
                    ax.text(icp[0], icp[1], 'N%d' % (jj + 1), fontsize=10)
            if ci is not None:
                ax.plot(ci[:,0], ci[:,1], 'b--', alpha=0.6)

        ax.set_aspect('equal')

        if ret_ax:
            return ax

        else:
            plt.show()

    def draw_basis(self):
        """
        Draw B-spline curve.
        """
        plt.figure()
        plt.plot(self.t,self.basis)
        plt.legend(['b%d' % (ii + 1) for ii in range(self.basis.shape[1])])
        plt.show()

    def approximate(self, coors, ncp=None, knot_type='clamped',
                    knots=None, alpha=0.5,
                    do_eval=False, do_param_correction=False):
        """
        Approximate set of points by the B-spline curve.

        Parameters
        ----------
        coors : array
            The coordinates of the approximated points.
        ncp : int
            The number of control points.
        knot_type : str
            The knot vector type.
        knots : array
            The knot vector.
        alpha : float
            The parameter vector distribution:
                1.0 = chordal
                0.5 = centripetal
        do_eval : bool
            Evaluate the curve coordinates?
        do_param_correction : bool
            Perform parametric corrections to improve the approximation?
        """
        coors = to_ndarray(coors)
        dg = self.degree

        if ncp is not None:
            if self.is_cyclic:
                ncp += dg
            self.ncp = ncp
            self.make_knot_vector(knot_type)

        if knots is not None:
            self.knots = knots
            self.knot_type = 'userdef'
            ncp = len(knots) - dg - 1
            self.ncp = ncp

        # param vector
        dist = nm.linalg.norm(coors[:-1,:] - coors[1:,:], axis=1)
        dista = nm.power(dist, alpha)
        self.t = nm.zeros((coors.shape[0],), dtype=nm.float64)
        self.t[1:] += dista.cumsum() / dista.sum()
        self.t[-1] -= nm_f64_eps

        while True:
            self.basis = self.basis_function_dg(dg, self.t,
                                                self.knots, ncp)

            A = nm.dot(self.basis.T, self.basis)
            b = nm.dot(self.basis.T, coors)
            # cyclic spline
            if self.is_cyclic:
                nred = ncp - dg
                R = nm.zeros((ncp, nred), dtype=nm.float64)
                for ii in range(nred):
                    R[ii,ii] = 1.0

                for ii in range(self.degree):
                    R[nred + ii,ii] = 1.0

                A = nm.dot(R.T, nm.dot(A, R))
                b = nm.dot(R.T, b)

                self.cp_coors = nm.dot(R, nm.dot(nm.linalg.inv(A), b))

            else:
                self.cp_coors = nm.dot(nm.linalg.inv(A), b)

            self.approx_coors = coors

            if not do_param_correction:
                break

        if do_eval:
            self.curve_coors = nm.dot(self.basis, self.cp_coors)

    def set_approx_points(self, coors):
        """
        Set the coordinates of approximated points.

        Parameters
        ----------
        coors : array
            The coordinates of approximated points.
        """
        self.approx_coors = to_ndarray(coors)

class BSplineSurf(Struct):
    """
    B-spline surface representation
    """

    def __init__(self, degree=(3,3), is_cyclic=(False, False)):
        """
        Initialize B-spline class.

        Parameters
        ----------
        degree : tuple of int
            The degree of the spline functions.
        is_cyclic : tuple of bool
            Cyclic splines?.
        """

        self.splines = [None, None]
        for ii in range(2):
            self.splines[ii] = BSpline(degree[ii], is_cyclic=is_cyclic[ii])

        self.surf_coors = None
        self.cp_coors = None
        self.approx_coors = None

    def set_control_points(self, coors, cyclic_form=False):
        """
        Set the B-spline control points.

        Parameters
        ----------
        coors : array
            The coordinates of unique control points.
        cyclic_form : bool
            Are the control points in the cyclic form?
        """
        coors = to_ndarray(coors)

        if self.splines[0].is_cyclic and not cyclic_form:
            coors = nm.vstack((coors, coors[0:self.splines[0].degree,:,:]))

        if self.splines[1].is_cyclic and not cyclic_form:
            coors = nm.hstack((coors, coors[:,0:self.splines[1].degree,:]))

        self.cp_coors = coors
        for ii in range(2):
            self.splines[ii].ncp = coors.shape[ii]

    def get_control_points(self):
        """
        Get the B-spline surface control points.

        Returns
        -------
        coors : array
            The coordinates of control points.
        """

        aux = self.cp_coors
        if self.splines[0].is_cyclic:
            aux = aux[:-self.splines[0].degree,:,:]

        if self.splines[1].is_cyclic:
            aux = aux[:,:-self.splines[1].degree,:]

        return aux

    def make_knot_vector(self, knot_type=('clamped', 'clamped'),
                         knot_data=(None, None)):
        """
        Create a knot vector of the requested type.

        Parameters
        ----------
        knot_type : tuple of str
            The knot vector types.
        knot_data : tuple of ANY
            The extra knot data.
        """
        for ii in range(2):
            self.splines[ii].make_knot_vector(knot_type[ii], knot_data[ii])

    def set_param_n(self, n=(100, 100)):
        """
        Generate the B-spline parametric vector using the number of steps.

        Parameters
        ----------
        n : tuple of array
            The number of steps in the B-spline parametric vectors.
        """
        for ii in range(2):
            self.splines[ii].set_param_n(n[ii])

    def set_approx_points(self, coors):
        """
        Set the coordinates of approximated points.

        Parameters
        ----------
        coors : array
            The coordinates of approximated points.
        """
        self.approx_coors = to_ndarray(coors)

    def eval(self, t=(None, None), cp_coors=None):
        """
        Evaluate the coordinates of the bpsline curve.

        Parameters
        ----------
        t : tuple of array
            The parametric vector of the B-splines.
        cp_coors : array
            The coordinates of the control points.
        """
        if cp_coors is not None:
            self.set_control_points(cp_coors)

        for ii in range(2):
            self.splines[ii].eval_basis(t[ii])

        nt = (len(self.splines[0].t), len(self.splines[1].t))
        ncp = (self.splines[0].ncp, self.splines[1].ncp)
        aux = nm.zeros((nt[0], ncp[1], 3), dtype=nm.float64)
        for ii in range(ncp[1]):
            aux[:,ii,:] = nm.dot(self.splines[0].basis, self.cp_coors[:,ii,:])

        self.surf_coors = nm.zeros(nt + (3,), dtype=nm.float64)
        for ii in range(nt[0]):
            self.surf_coors[ii,:,:] = nm.dot(self.splines[1].basis, aux[ii,:,:])

        return self.surf_coors

    def draw(self, ret_ax=False, ax=None):
        """
        Draw B-spline surface.

        Parameters
        ----------
        ret_ax : bool
            Return an axes object?
        ax : axes object
            The axes to which will be drawn.
        """

        if self.surf_coors is None:
            self.eval()

        fig = plt.figure()
        ax = Axes3D(fig)
        coors = self.surf_coors
        cs = coors.shape
        for ii in range(cs[0] - 1):
            for jj in range(cs[1] - 1):
                verts = nm.array([coors[ii,jj,:],
                                  coors[ii,jj + 1,:],
                                  coors[ii + 1,jj + 1,:],
                                  coors[ii + 1,jj,:]])

                quad = Poly3DCollection([verts],
                                        facecolors='gray', edgecolor='k',
                                        linewidths=0.2, alpha=0.5)
                ax.add_collection3d(quad)

        cp = self.cp_coors
        for ii in range(cp.shape[1]):
            ax.plot(cp[:,ii,0], cp[:,ii,1], cp[:,ii,2], 'ro--', linewidth=2.0)
        for ii in range(cp.shape[0]):
            ax.plot(cp[ii,:,0], cp[ii,:,1], cp[ii,:,2], 'ro--', linewidth=2.0)

        ax.set_aspect('equal')
        plt.show()

    def approximate(self, coors, ncp, do_eval=False):
        """
        Approximate set of points by the B-spline surface.

        Parameters
        ----------
        coors : array
            The coordinates of the approximated points.
        ncp : tuple of int
            The number of control points.
        """
        coors = to_ndarray(coors)

        nsc = coors.shape[0:2]
        aux = nm.zeros((nsc[0], ncp[1], 3), dtype=nm.float64)
        spl1 = self.splines[1]
        for ii in range(nsc[0]):
            spl1.approximate(coors[ii,...], ncp[1])
            aux[ii,...] = spl1.get_control_points()

        self.cp_coors = nm.zeros((ncp[0], ncp[1], 3), dtype=nm.float64)
        spl2 = self.splines[0]
        for ii in range(ncp[1]):
            spl2.approximate(aux[:,ii,:], ncp[0])
            self.cp_coors[:,ii,:] = spl2.get_control_points()

        self.approx_coors = coors

    def write_surface_vtk(self, filename, float_format='%.6f'):
        """
        Write the spline surface to VTK file.

        Parameters
        ----------
        filename: str
            Name of the VTK file.
        float_format: str
            Float formating.
        """
        coors = self.surf_coors
        cs0, cs1 = coors.shape[0:2]

        nquads = (cs0 - 1) * (cs1 - 1)
        quads = nm.zeros((nquads, 4), dtype=nm.int64)
        kk = 0
        for ii in range(cs0 - 1):
            offs = ii * cs1
            for jj in range(cs1 - 1):
                quads[kk] = nm.array([jj + offs,
                                      jj + offs + cs1,
                                      jj + 1 + offs + cs1,
                                      jj + 1 + offs])
                kk += 1

        f = open(filename, 'w')
        f.write('# vtk DataFile Version 2.0\n')
        f.write('spline surface\nASCII\nDATASET POLYDATA\n')

        ff3 = ' '.join([float_format] * 3) + '\n'
        f.write('POINTS %d float\n' % (cs0 * cs1))
        for ii in range(cs0):
            offs = ii * cs1
            for jj in range(cs1):
                f.write(ff3 % tuple(coors[ii,jj,:]))

        f.write('POLYGONS %d %d\n' % (nquads, nquads * 5))
        for ii in quads:
            f.write('4 %s\n' % (' '.join([str(jj) for jj in ii])))

        f.close()

    def write_control_polygon_vtk(self, filename, float_format='%.6f'):
        """
        Write the control polygon to VTK file.

        Parameters
        ----------
        filename: str
            Name of the VTK file.
        float_format: str
            Float formating.
        """
        coors = self.cp_coors
        cs0, cs1 = coors.shape[0:2]

        lines = []
        nlines = cs0 + cs1
        nlpts = 0
        for ii in range(cs0):
            lines.append(nm.arange(cs1) + ii * cs1)
            nlpts += cs1
        for ii in range(cs1):
            lines.append(nm.arange(cs0) * cs1 + ii)
            nlpts += cs0

        f = open(filename, 'w')
        f.write('# vtk DataFile Version 2.0\n')
        f.write('spline control polygon\nASCII\nDATASET POLYDATA\n')

        ff3 = ' '.join([float_format] * 3) + '\n'
        f.write('POINTS %d float\n' % (cs0 * cs1))
        for ii in range(cs0):
            for jj in range(cs1):
                f.write(ff3 % tuple(coors[ii,jj,:]))

        f.write('LINES %d %d\n' % (nlines, nlines + nlpts))
        for ii in lines:
            f.write('%d %s\n' % (len(ii), ' '.join([str(jj) for jj in ii])))

        f.close()

def get_2d_points(is3d=False):
    """
    Returns the set of points.

    Parameters
    ----------
    is3d : bool
        3D coordinates?
    """
    out = nm.array([(-0.87, 0.15, 0),
                    (-0.70, 0.54, 0),
                    (-0.32, 0.80, 0),
                    (0.15, 0.70, 0),
                    (0.37, 0.26, 0),
                    (0.70, -0.07, 0),
                    (0.67, -0.49, 0),
                    (0.07, -0.81, 0),
                    (-0.44, -0.72, 0),
                    (-0.80, -0.34, 0)])

    if is3d:
        return out

    else:
        return out[:,:2]

def approximation_example():
    """
    The example of using BSplineSurf for approximation
    of the surface given by the set of points.
    """
    # define sample points in 2D
    spts0 = get_2d_points(is3d=True)
    # define sample points in 3D
    spts = nm.array([spts0,
                     spts0 * 0.7 + nm.array([0.1,0,0.5]),
                     spts0 * 0.8 + nm.array([0.2,0,1.5]),
                     spts0 * 1.2 + nm.array([0.4,0,2.5])])

    cyclic=(False, True)
    spl1 = BSplineSurf((3, 3), is_cyclic=cyclic)
    spl1.approximate(spts, (4,8))
    cp = spl1.get_control_points()

    spls2 = BSplineSurf((3, 3), is_cyclic=cyclic)
    spls2.set_control_points(cp)
    spls2.make_knot_vector()
    spls2.set_param_n((12, 24))
    spls2.eval()
    spls2.draw()

def simple_example():
    """
    The example of using B-spline class.
    """
    # define control points in 2D
    cp = get_2d_points()

    spl = BSpline(3, is_cyclic=True)
    spl.set_control_points(cp)
    spl.make_knot_vector()
    spl.set_param_n(150)

    spl.insert_knot(0.7)
    spl.insert_knot(0.7)
    spl.insert_knot(0.7)
    spl.eval()
    spl.draw()

def main(argv):
    # simple_example()
    approximation_example()

if __name__ == '__main__':
    main(sys.argv)



import numpy as nm

from sfepy.base.base import assert_
from sfepy.linalg import dot_sequences
from sfepy.terms.terms import Term, terms
from sfepy.terms.terms_th import THTerm, ETHTerm

class DotProductVolumeTerm(Term):
    r"""
    Volume :math:`L^2(\Omega)` weighted dot product for both scalar and vector
    fields. Can be evaluated. Can use derivatives.

    :Definition:

    .. math::
        \int_\Omega q p \mbox{ , } \int_\Omega \ul{v} \cdot \ul{u}
        \mbox{ , }
        \int_\Omega p r \mbox{ , } \int_\Omega \ul{u} \cdot \ul{w} \\
        \int_\Omega c q p \mbox{ , } \int_\Omega c \ul{v} \cdot \ul{u}
        \mbox{ , }
        \int_\Omega c p r \mbox{ , } \int_\Omega c \ul{u} \cdot \ul{w} \\
        \int_\Omega \ul{v} \cdot (\ull{M} \ul{u})
        \mbox{ , }
        \int_\Omega \ul{u} \cdot (\ull{M} \ul{w})

    :Arguments 1:
        - material : :math:`c` or :math:`\ull{M}` (optional)
        - virtual  : :math:`q` or :math:`\ul{v}`
        - state    : :math:`p` or :math:`\ul{u}`

    :Arguments 2:
        - material    : :math:`c` or :math:`\ull{M}` (optional)
        - parameter_1 : :math:`p` or :math:`\ul{u}`
        - parameter_2 : :math:`r` or :math:`\ul{w}`
    """
    name = 'dw_volume_dot'
    arg_types = (('opt_material', 'virtual', 'state'),
                 ('opt_material', 'parameter_1', 'parameter_2'))
    arg_shapes = [{'opt_material' : '1, 1', 'virtual' : (1, 'state'),
                   'state' : 1, 'parameter_1' : 1, 'parameter_2' : 1},
                  {'opt_material' : None},
                  {'opt_material' : '1, 1', 'virtual' : ('D', 'state'),
                   'state' : 'D', 'parameter_1' : 'D', 'parameter_2' : 'D'},
                  {'opt_material' : 'D, D'},
                  {'opt_material' : None}]
    modes = ('weak', 'eval')

    @staticmethod
    def dw_dot(out, mat, val_qp, vgeo, sgeo, fun, fmode):
        status = fun(out, mat, val_qp, vgeo, sgeo, fmode)
        return status

    @staticmethod
    def d_dot(out, mat, val1_qp, val2_qp, geo):
        if mat is None:
            if val1_qp.shape[2] > 1:
                if val2_qp.shape[2] == 1:
                    aux = dot_sequences(val1_qp, geo.normal, mode='ATB')
                    vec = dot_sequences(aux, val2_qp, mode='AB')

                else:
                    vec = dot_sequences(val1_qp, val2_qp, mode='ATB')

            else:
                vec = val1_qp * val2_qp

        elif mat.shape[-1] == 1:
            if val1_qp.shape[2] > 1:
                vec = mat * dot_sequences(val1_qp, val2_qp, mode='ATB')

            else:
                vec = mat * val1_qp * val2_qp

        else:
            aux = dot_sequences(mat, val2_qp, mode='AB')
            vec = dot_sequences(val1_qp, aux, mode='ATB')

        status = geo.integrate(out, vec)

        return status

    def get_fargs(self, mat, virtual, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vgeo, _ = self.get_mapping(virtual)

        if mode == 'weak':
            if mat is None:
                n_cell, n_qp, dim, n_n, n_c = self.get_data_shape(state)
                mat = nm.ones((n_cell, n_qp, 1, 1), dtype=nm.float64)

            sgeo, _ = self.get_mapping(state)

            if diff_var is None:
                val_qp = self.get(state, 'val')
                fmode = 0

            else:
                val_qp = nm.array([0], ndmin=4, dtype=nm.float64)
                fmode = 1

            if state.n_components > 1:
                if ((self.integration == 'volume')
                    or (virtual.n_components > 1)):
                    fun = terms.dw_volume_dot_vector

                else:
                    fun = terms.dw_surface_s_v_dot_n

            else:
                if ((self.integration == 'volume')
                    or (virtual.n_components == 1)):
                    fun = terms.dw_volume_dot_scalar

                else:
                    fun = terms.dw_surface_v_dot_n_s

            return mat, val_qp, vgeo, sgeo, fun, fmode

        elif mode == 'eval':
            val1_qp = self.get(virtual, 'val')
            val2_qp = self.get(state, 'val')

            return mat, val1_qp, val2_qp, vgeo

        else:
            raise ValueError('unsupported evaluation mode in %s! (%s)'
                             % (self.name, mode))

    def get_eval_shape(self, mat, virtual, state,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_cell, n_qp, dim, n_n, n_c = self.get_data_shape(state)

        return (n_cell, 1, 1, 1), state.dtype

    def set_arg_types(self):
        if self.mode == 'weak':
            self.function = self.dw_dot

        else:
            self.function = self.d_dot

class DotProductSurfaceTerm(DotProductVolumeTerm):
    r"""
    Surface :math:`L^2(\Gamma)` dot product for both scalar and vector
    fields. If either virtual or state variable is a vector, the orientation
    of the normal vectors is outwards to the parent region of the virtual
    variable.

    :Definition:

    .. math::
        \int_\Gamma q p \mbox{ , } \int_\Gamma \ul{v} \cdot \ul{u}
        \mbox{ , }
        \int_\Gamma \ul{v} \cdot \ul{n} p \mbox{ , }
        \int_\Gamma q \ul{n} \cdot \ul{u} \mbox{ , }
        \int_\Gamma p r \mbox{ , } \int_\Gamma \ul{u} \cdot \ul{w}
        \mbox{ , } \int_\Gamma \ul{w} \cdot \ul{n} p \\
        \int_\Gamma c q p \mbox{ , } \int_\Gamma c \ul{v} \cdot \ul{u}
        \mbox{ , }
        \int_\Gamma c p r \mbox{ , } \int_\Gamma c \ul{u} \cdot \ul{w} \\
        \int_\Gamma \ul{v} \cdot \ull{M} \cdot \ul{u}
        \mbox{ , }
        \int_\Gamma \ul{u} \cdot \ull{M} \cdot \ul{w}

    :Arguments 1:
        - material : :math:`c` or :math:`\ull{M}` (optional)
        - virtual  : :math:`q` or :math:`\ul{v}`
        - state    : :math:`p` or :math:`\ul{u}`

    :Arguments 2:
        - material    : :math:`c` or :math:`\ull{M}` (optional)
        - parameter_1 : :math:`p` or :math:`\ul{u}`
        - parameter_2 : :math:`r` or :math:`\ul{w}`
    """
    name = 'dw_surface_dot'
    arg_types = (('opt_material', 'virtual', 'state'),
                 ('opt_material', 'parameter_1', 'parameter_2'))
    arg_shapes = [{'opt_material' : '1, 1', 'virtual' : (1, 'state'),
                   'state' : 1, 'parameter_1' : 1, 'parameter_2' : 1},
                  {'opt_material' : None},
                  {'opt_material' : '1, 1', 'virtual' : (1, None),
                   'state' : 'D'},
                  {'opt_material' : None},
                  {'opt_material' : '1, 1', 'virtual' : ('D', None),
                   'state' : 1},
                  {'opt_material' : None},
                  {'opt_material' : '1, 1', 'virtual' : ('D', 'state'),
                   'state' : 'D', 'parameter_1' : 'D', 'parameter_2' : 'D'},
                  {'opt_material' : 'D, D'},
                  {'opt_material' : None}]
    modes = ('weak', 'eval')
    integration = 'surface'

class BCNewtonTerm(DotProductSurfaceTerm):
    r"""
    Newton boundary condition term.

    :Definition:

    .. math::
        \int_{\Gamma} \alpha q (p - p_{\rm outer})

    :Arguments:
        - material_1 : :math:`\alpha`
        - material_2 : :math:`p_{\rm outer}`
        - virtual    : :math:`q`
        - state      : :math:`p`
    """
    name = 'dw_bc_newton'
    arg_types = ('material_1', 'material_2', 'virtual', 'state')
    arg_shapes = {'material_1' : '1, 1', 'material_2' : '1, 1',
                  'virtual' : (1, 'state'), 'state' : 1}
    mode = 'weak'

    def get_fargs(self, alpha, p_outer, virtual, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        fargs = DotProductSurfaceTerm.get_fargs(self, alpha, virtual, state,
                                                mode, term_mode, diff_var,
                                                **kwargs)
        fargs = fargs[:1] + (fargs[1] - p_outer,) + fargs[2:]

        return fargs

class DotSProductVolumeOperatorWTHTerm(THTerm):
    r"""
    Fading memory volume :math:`L^2(\Omega)` weighted dot product for
    scalar fields. Can use derivatives.

    :Definition:

    .. math::
        \int_\Omega \left [\int_0^t \Gcal(t-\tau) p(\tau) \difd{\tau} \right] q

    :Arguments:
        - ts       : :class:`TimeStepper` instance
        - material : :math:`\Gcal(\tau)`
        - virtual  : :math:`q`
        - state    : :math:`p`
    """
    name = 'dw_volume_dot_w_scalar_th'
    arg_types = ('ts', 'material', 'virtual', 'state')
    arg_shapes = {'material' : '.: N, 1, 1',
                  'virtual' : (1, 'state'), 'state' : 1}

    function = staticmethod(terms.dw_volume_dot_scalar)

    def get_fargs(self, ts, mats, virtual, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(state)

        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(state)

        if diff_var is None:
            def iter_kernel():
                for ii, mat in enumerate(mats):
                    val_qp = self.get(state, 'val', step=-ii)
                    mat = nm.tile(mat, (n_el, n_qp, 1, 1))
                    yield ii, (ts.dt * mat, val_qp, vg, vg, 0)
            fargs = iter_kernel

        else:
            val_qp = nm.array([0], ndmin=4, dtype=nm.float64)
            mat = nm.tile(mats[0], (n_el, n_qp, 1, 1))
            fargs = ts.dt * mat, val_qp, vg, vg, 1

        return fargs

class DotSProductVolumeOperatorWETHTerm(ETHTerm):
    r"""
    Fading memory volume :math:`L^2(\Omega)` weighted dot product for
    scalar fields. This term has the same definition as
    dw_volume_dot_w_scalar_th, but assumes an exponential approximation of
    the convolution kernel resulting in much higher efficiency. Can use
    derivatives.

    :Definition:

    .. math::
        \int_\Omega \left [\int_0^t \Gcal(t-\tau) p(\tau) \difd{\tau} \right] q

    :Arguments:
        - ts         : :class:`TimeStepper` instance
        - material_0 : :math:`\Gcal(0)`
        - material_1 : :math:`\exp(-\lambda \Delta t)` (decay at :math:`t_1`)
        - virtual    : :math:`q`
        - state      : :math:`p`
    """
    name = 'dw_volume_dot_w_scalar_eth'
    arg_types = ('ts', 'material_0', 'material_1', 'virtual', 'state')
    arg_shapes = {'material_0' : '1, 1', 'material_1' : '1, 1',
                  'virtual' : (1, 'state'), 'state' : 1}

    function = staticmethod(terms.dw_volume_dot_scalar)

    def get_fargs(self, ts, mat0, mat1, virtual, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _, key = self.get_mapping(state, return_key=True)

        if diff_var is None:
            val_qp = self.get(state, 'val')

            key += tuple(self.arg_names[ii] for ii in [1, 2, 4])
            data = self.get_eth_data(key, state, mat1, val_qp)

            fargs = (ts.dt * mat0, data.history + data.values, vg, vg, 0)

        else:
            aux = nm.array([0], ndmin=4, dtype=nm.float64)
            fargs = ts.dt * mat0, aux, vg, vg, 1

        return fargs

class VectorDotGradScalarTerm(Term):
    r"""
    Volume dot product of a vector and a gradient of scalar.
    Can be evaluated.

    :Definition:

    .. math::
        \int_{\Omega} \ul{v} \cdot \nabla p \mbox{ , }
        \int_{\Omega} \ul{u} \cdot \nabla q \\
        \int_{\Omega} c \ul{v} \cdot \nabla p \mbox{ , }
        \int_{\Omega} c \ul{u} \cdot \nabla q \\
        \int_{\Omega} \ul{v} \cdot (\ull{M} \nabla p) \mbox{ , }
        \int_{\Omega} \ul{u} \cdot (\ull{M} \nabla q)

    :Arguments 1:
        - material : :math:`c` or :math:`\ull{M}` (optional)
        - virtual  : :math:`\ul{v}`
        - state    : :math:`p`

    :Arguments 2:
        - material : :math:`c` or :math:`\ull{M}` (optional)
        - state    : :math:`\ul{u}`
        - virtual  : :math:`q`

    :Arguments 3:
        - material    : :math:`c` or :math:`\ull{M}` (optional)
        - parameter_v : :math:`\ul{u}`
        - parameter_s : :math:`p`
    """
    name = 'dw_v_dot_grad_s'
    arg_types = (('opt_material', 'virtual', 'state'),
                 ('opt_material', 'state', 'virtual'),
                 ('opt_material', 'parameter_v', 'parameter_s'))
    arg_shapes = [{'opt_material' : '1, 1',
                   'virtual/v_weak' : ('D', None), 'state/v_weak' : 1,
                   'virtual/s_weak' : (1, None), 'state/s_weak' : 'D',
                   'parameter_v' : 'D', 'parameter_s' : 1},
                  {'opt_material' : 'D, D'},
                  {'opt_material' : None}]
    modes = ('v_weak', 's_weak', 'eval')

    def get_fargs(self, coef, vvar, svar,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(vvar)
        if coef is None:
            coef = nm.ones((1, n_qp, 1, 1), dtype=nm.float64)

        if mode == 'weak':
            if self.mode == 'v_weak':
                qp_var, qp_name = svar, 'grad'

            else:
                qp_var, qp_name = vvar, 'val'

            vvg, _ = self.get_mapping(vvar)
            svg, _ = self.get_mapping(svar)

            if diff_var is None:
                val_qp = self.get(qp_var, qp_name)
                fmode = 0

            else:
                val_qp = nm.array([0], ndmin=4, dtype=nm.float64)
                fmode = 1

            return coef, val_qp, vvg, svg, fmode

        elif mode == 'eval':
            vvg, _ = self.get_mapping(vvar)

            grad = self.get(svar, 'grad')
            val = self.get(vvar, 'val')

            return coef, grad, val, vvg

        else:
            raise ValueError('unsupported evaluation mode in %s! (%s)'
                             % (self.name, mode))

    def get_eval_shape(self, coef, vvar, svar,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(vvar)

        return (n_el, 1, 1, 1), vvar.dtype

    def set_arg_types(self):
        self.function = {
            'v_weak' : terms.dw_v_dot_grad_s_vw,
            's_weak' : terms.dw_v_dot_grad_s_sw,
            'eval' : DotProductVolumeTerm.d_dot,
        }[self.mode]

class VectorDotScalarTerm(Term):
    r"""
    Volume dot product of a vector and a scalar.
    Can be evaluated.

    :Definition:

    .. math::
        \int_{\Omega} \ul{v} \cdot \ul{m} p \mbox{ , }
        \int_{\Omega} \ul{u} \cdot \ul{m} q\\

    :Arguments 1:
        - material : :math:`\ul{m}`
        - virtual  : :math:`\ul{v}`
        - state    : :math:`p`

    :Arguments 2:
        - material : :math:`\ul{m}`
        - state    : :math:`\ul{u}`
        - virtual  : :math:`q`

    :Arguments 3:
        - material    : :math:`\ul{m}`
        - parameter_v : :math:`\ul{u}`
        - parameter_s : :math:`p`
    """
    name = 'dw_vm_dot_s'
    arg_types = (('material', 'virtual', 'state'),
                 ('material', 'state', 'virtual'),
                 ('material', 'parameter_v', 'parameter_s'))
    arg_shapes = [{'material' : 'D, 1',
                   'virtual/v_weak' : ('D', None), 'state/v_weak' : 1,
                   'virtual/s_weak' : (1, None), 'state/s_weak' : 'D',
                   'parameter_v' : 'D', 'parameter_s' : 1}]
    modes = ('v_weak', 's_weak', 'eval')

    @staticmethod
    def dw_dot(out, mat, val_qp, bfve, bfsc, geo, fmode):

        nel, nqp, dim, nc = mat.shape
        nen = bfve.shape[3]

        status1 = 0
        if fmode in [0, 1, 3]:
            aux = nm.zeros((nel, nqp, dim * nen, nc), dtype=nm.float64)
            status1 = terms.actBfT(aux, bfve, mat)

        if fmode == 0:
            status2 = terms.mulAB_integrate(out, aux, val_qp, geo, 'AB')

        if fmode == 1:
            status2 = terms.mulAB_integrate(out, aux, bfsc, geo, 'AB')

        if fmode == 2:
            aux = (bfsc * dot_sequences(mat, val_qp,
                                        mode='ATB')).transpose((0,1,3,2))
            status2 = geo.integrate(out, nm.ascontiguousarray(aux))

        if fmode == 3:
            status2 = terms.mulAB_integrate(out, bfsc, aux, geo, 'ATBT')

        return status1 and status2

    @staticmethod
    def d_dot(out, mat, val1_qp, val2_qp, geo):
        v1, v2 = (val1_qp, val2_qp) if val1_qp.shape[2] > 1 \
                 else (val2_qp, val1_qp)
        aux = dot_sequences(v1, mat, mode='ATB')
        vec = dot_sequences(aux, v2, mode='AB')
        status = geo.integrate(out, vec)

        return status

    def get_fargs(self, coef, vvar, svar,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(vvar)

        coef = coef.reshape(coef.shape[:2] + (dim, 1))

        if mode == 'weak':
            vgv, _ = self.get_mapping(vvar)
            vgs, _ = self.get_mapping(svar)

            bfve = vgv.bf
            bfsc = vgs.bf

            if self.mode == 'v_weak':
                qp_var, geo, fmode = svar, vgv, 0

            else:
                qp_var, geo, fmode = vvar, vgs, 2
                bfve, bfsc = bfsc, bfve

            if diff_var is None:
                val_qp = self.get(qp_var, 'val')

            else:
                val_qp = (nm.array([0], ndmin=4, dtype=nm.float64), 1)
                fmode += 1

            return coef, val_qp, bfve, bfsc, geo, fmode

        elif mode == 'eval':
            vvg, _ = self.get_mapping(vvar)
            vals = self.get(svar, 'val')
            valv = self.get(vvar, 'val')

            return coef, vals, valv, vvg

        else:
            raise ValueError('unsupported evaluation mode in %s! (%s)'
                             % (self.name, mode))

    def get_eval_shape(self, coef, vvar, svar,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(vvar)

        return (n_el, 1, 1, 1), vvar.dtype

    def set_arg_types(self):
        self.function = {
            'v_weak' : self.dw_dot,
            's_weak' : self.dw_dot,
            'eval' : self.d_dot,
        }[self.mode]

class ScalarDotGradIScalarTerm(Term):
    r"""
    Dot product of a scalar and the :math:`i`-th component of gradient of a
    scalar. The index should be given as a 'special_constant' material
    parameter.

    :Definition:

    .. math::
        Z^i = \int_{\Omega} q \nabla_i p

    :Arguments:
        - material : :math:`i`
        - virtual  : :math:`q`
        - state    : :math:`p`
    """
    name = 'dw_s_dot_grad_i_s'
    arg_types = ('material', 'virtual', 'state')
    arg_shapes = {'material' : '.: 1, 1', 'virtual' : (1, 'state'), 'state' : 1}

    @staticmethod
    def dw_fun(out, bf, vg, grad, idx, fmode):
        cc = nm.ascontiguousarray
        bft = cc(nm.tile(bf, (out.shape[0], 1, 1, 1)))

        if fmode == 0:
            status = terms.mulAB_integrate(out, bft,
                                           cc(grad[..., idx:idx+1, :]), vg,
                                           mode='ATB')

        else:
            status = terms.mulAB_integrate(out, bft,
                                           cc(vg.bfg[:,:,idx:(idx + 1),:]), vg,
                                           mode='ATB')

        return status

    def get_fargs(self, material, virtual, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        if mode == 'weak':
            if diff_var is None:
                grad = self.get(state, 'grad')
                fmode = 0

            else:
                grad = nm.array([0], ndmin=4, dtype=nm.float64)
                fmode = 1

            vg, _ = self.get_mapping(virtual)
            vgs, _ = self.get_mapping(state)

            idx = int(material)

            return vgs.bf, vg, grad, idx, fmode

        else:
            raise ValueError('unsupported evaluation mode in %s! (%s)'
                             % (self.name, mode))

    def set_arg_types(self):
        self.function = self.dw_fun

class ScalarDotMGradScalarTerm(Term):
    r"""
    Volume dot product of a scalar gradient dotted with a material vector with
    a scalar.

    :Definition:

    .. math::
        \int_{\Omega} q \ul{y} \cdot \nabla p \mbox{ , }
        \int_{\Omega} p \ul{y} \cdot \nabla q

    :Arguments 1:
        - material : :math:`\ul{y}`
        - virtual  : :math:`q`
        - state    : :math:`p`

    :Arguments 2:
        - material : :math:`\ul{y}`
        - state    : :math:`p`
        - virtual  : :math:`q`
    """
    name = 'dw_s_dot_mgrad_s'
    arg_types = (('material', 'virtual', 'state'),
                 ('material', 'state', 'virtual'))
    arg_shapes = [{'material' : 'D, 1',
                   'virtual/grad_state' : (1, None),
                   'state/grad_state' : 1,
                   'virtual/grad_virtual' : (1, None),
                   'state/grad_virtual' : 1}]
    modes = ('grad_state', 'grad_virtual')

    @staticmethod
    def function(out, out_qp, geo, fmode):
        status = geo.integrate(out, out_qp)
        return status

    def get_fargs(self, mat, var1, var2,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg1, _ = self.get_mapping(var1)
        vg2, _ = self.get_mapping(var2)

        if diff_var is None:
            if self.mode == 'grad_state':
                geo = vg1
                bf_t = vg1.bf.transpose((0, 1, 3, 2))
                val_qp = self.get(var2, 'grad')
                out_qp = bf_t * dot_sequences(mat, val_qp, 'ATB')

            else:
                geo = vg2
                val_qp = self.get(var1, 'val')
                out_qp = dot_sequences(vg2.bfg, mat, 'ATB') * val_qp

            fmode = 0

        else:
            if self.mode == 'grad_state':
                geo = vg1
                bf_t = vg1.bf.transpose((0, 1, 3, 2))
                out_qp = bf_t * dot_sequences(mat, vg2.bfg, 'ATB')

            else:
                geo = vg2
                out_qp = dot_sequences(vg2.bfg, mat, 'ATB') * vg1.bf

            fmode = 1

        return out_qp, geo, fmode



from __future__ import absolute_import
import numpy as nm
import scipy.sparse as sps

from sfepy.base.testing import TestCommon

class Test(TestCommon):

    @staticmethod
    def from_conf(conf, options):
        conn = nm.array([[0, 1, 2],
                         [2, 3, 4]], dtype=nm.int32)

        num = conn.max() + 1

        iels = nm.array([0, 1], dtype=nm.int32)

        vec_in_els = nm.array([[1, 1, 1], [2, 2, 2]], dtype=nm.float64)
        vec_in_els.shape = (2, 1, 1, 3)

        mtx_in_els = nm.array([nm.ones((3, 3)), 2 * nm.ones((3, 3))],
                              dtype=nm.float64)
        mtx_in_els.shape = (2, 1, 3, 3)

        return Test(conn=conn, num=num, iels=iels,
                    vec_in_els=vec_in_els, mtx_in_els=mtx_in_els,
                    conf=conf, options=options)

    def test_assemble_vector(self):
        from sfepy.discrete.common.extmods.assemble import assemble_vector

        vec = nm.zeros(self.num, dtype=nm.float64)

        assemble_vector(vec, self.vec_in_els, self.iels, 1, self.conn)

        aux = nm.array([1, 1, 3, 2, 2], dtype=nm.float64)

        self.report('assembled: %s' % vec)
        self.report('expected: %s' % aux)
        ok = self.compare_vectors(vec, aux,
                                  label1='assembled',
                                  label2='expected')
        return ok

    def test_assemble_vector_complex(self):
        from sfepy.discrete.common.extmods.assemble import assemble_vector_complex

        vec = nm.zeros(self.num, dtype=nm.complex128)
        vec_in_els = self.vec_in_els.astype(nm.complex128) * (2 - 3j)

        assemble_vector_complex(vec, vec_in_els, self.iels, 1, self.conn)

        aux = nm.array([2-3j, 2-3j, 6-9j, 4-6j, 4-6j],
                       dtype=nm.complex128)

        self.report('assembled: %s' % vec)
        self.report('expected: %s' % aux)
        ok = self.compare_vectors(vec, aux,
                                  label1='assembled',
                                  label2='expected')
        return ok

    def test_assemble_matrix(self):
        from sfepy.discrete.common.extmods.assemble import assemble_matrix

        mtx = sps.csr_matrix(nm.ones((self.num, self.num),
                                     dtype=nm.float64))
        mtx.data[:] = 0.0

        assemble_matrix(mtx.data, mtx.indptr, mtx.indices, self.mtx_in_els,
                        self.iels, 1, self.conn, self.conn)

        aux = nm.array([[1, 1, 1, 0, 0],
                        [1, 1, 1, 0, 0],
                        [1, 1, 3, 2, 2],
                        [0, 0, 2, 2, 2],
                        [0, 0, 2, 2, 2]], dtype=nm.float64)

        self.report('assembled:\n%s' % mtx.toarray())
        self.report('expected:\n%s' % aux)
        ok = self.compare_vectors(mtx, aux,
                                  label1='assembled',
                                  label2='expected')
        return ok

    def test_assemble_matrix_complex(self):
        from sfepy.discrete.common.extmods.assemble import assemble_matrix_complex

        mtx = sps.csr_matrix(nm.ones((self.num, self.num),
                                     dtype=nm.complex128))
        mtx.data[:] = 0.0
        mtx_in_els = self.mtx_in_els.astype(nm.complex128) * (2 - 3j)

        assemble_matrix_complex(mtx.data, mtx.indptr, mtx.indices, mtx_in_els,
                                self.iels, 1, self.conn, self.conn)

        aux = nm.array([[2-3j, 2-3j, 2-3j, 0+0j, 0+0j],
                        [2-3j, 2-3j, 2-3j, 0+0j, 0+0j],
                        [2-3j, 2-3j, 6-9j, 4-6j, 4-6j],
                        [0+0j, 0+0j, 4-6j, 4-6j, 4-6j],
                        [0+0j, 0+0j, 4-6j, 4-6j, 4-6j]], dtype=nm.complex128)

        self.report('assembled:\n%s' % mtx.toarray())
        self.report('expected:\n%s' % aux)
        ok = self.compare_vectors(mtx, aux,
                                  label1='assembled',
                                  label2='expected')
        return ok



from __future__ import absolute_import
from copy import copy

import numpy as nm

from sfepy.base.base import output, get_default, OneTypeList, Struct, basestr
from sfepy.discrete import Equations, Variables, Region, Integral, Integrals
from sfepy.discrete.common.fields import setup_extra_data
import six

def apply_ebc_to_matrix(mtx, ebc_rows, epbc_rows=None):
    """
    Apply E(P)BC to matrix rows: put 1 to the diagonal for EBC DOFs, 1 to the
    diagonal for master EPBC DOFs, -1 to the [master, slave] entries. It is
    assumed, that the matrix contains zeros in EBC and master EPBC DOFs rows
    and columns.
    """
    data, prows, cols = mtx.data, mtx.indptr, mtx.indices
    # Does not change the sparsity pattern.
    for ir in ebc_rows:
        for ic in range(prows[ir], prows[ir + 1]):
            if (cols[ic] == ir):
                data[ic] = 1.0

    if epbc_rows is not None:
        master, slave = epbc_rows

        # Changes sparsity pattern in-place - allocates new entries! The master
        # DOFs are not allocated by Equations.create_matrix_graph(), see
        # create_adof_conns().
        mtx[master, master] = 1.0
        mtx[master, slave] = -1.0

##
# 02.10.2007, c
class Evaluator(Struct):
    """
    This class provides the functions required by a nonlinear solver for a
    given problem.
    """

    def __init__(self, problem, matrix_hook=None):
        Struct.__init__(self, problem=problem, matrix_hook=matrix_hook)

    @staticmethod
    def new_ulf_iteration(problem, nls, vec, it, err, err0):
        vec = problem.equations.make_full_vec(vec)
        problem.equations.set_variables_from_state(vec)

        upd_vars = problem.conf.options.get('mesh_update_variables', None)
        for varname in upd_vars:
            try:
                state = problem.equations.variables[varname]
            except IndexError:
                msg = 'variable "%s" does not exist!' % varname
                raise KeyError( msg )

        nods = state.field.get_dofs_in_region(state.field.region, merge=True)
        coors = problem.domain.get_mesh_coors().copy()
        coors[nods, :] += state().reshape(len(nods), state.n_components)

        if len(state.field.mappings0) == 0:
            state.field.save_mappings()

        state.field.clear_mappings()
        problem.set_mesh_coors(coors, update_fields=False, actual=True,
                               clear_all=False)

    def eval_residual(self, vec, is_full=False):
        if not is_full and self.problem.active_only:
            vec = self.make_full_vec(vec)

        vec_r = self.problem.equations.eval_residuals(vec)
        if self.matrix_hook is not None:
            vec_r = self.matrix_hook(vec_r, self.problem, call_mode='residual')

        if self.problem.equations.variables.has_lcbc:
            mtx_lcbc = self.problem.equations.get_lcbc_operator()

            vec_rr = mtx_lcbc.T * vec_r
            if self.matrix_hook is not None:
                vec_rr = self.matrix_hook(vec_rr, self.problem,
                                          call_mode='lcbc_residual')
            vec_r = vec_rr

        return vec_r

    def eval_tangent_matrix(self, vec, mtx=None, is_full=False):
        if isinstance(vec, basestr) and vec == 'linear':
            return get_default(mtx, self.problem.mtx_a)

        if not is_full and self.problem.active_only:
            vec = self.make_full_vec( vec )

        pb = self.problem
        if mtx is None:
            mtx = pb.mtx_a
        mtx = pb.equations.eval_tangent_matrices(vec, mtx)

        if not pb.active_only:
            apply_ebc_to_matrix(mtx, *pb.get_ebc_indices())

        if self.matrix_hook is not None:
            mtx = self.matrix_hook(mtx, pb, call_mode='basic')

        if self.problem.equations.variables.has_lcbc:
            mtx_lcbc = self.problem.equations.get_lcbc_operator()

            mtx_r = mtx_lcbc.T * mtx * mtx_lcbc
            mtx_r = mtx_r.tocsr()
            mtx_r.sort_indices()

            if self.matrix_hook is not None:
                mtx_r = self.matrix_hook(mtx_r, self.problem, call_mode='lcbc')

            mtx = mtx_r

        return mtx

    def make_full_vec(self, vec):
        return self.problem.equations.make_full_vec(vec)

def create_evaluable(expression, fields, materials, variables, integrals,
                     regions=None,
                     ebcs=None, epbcs=None, lcbcs=None,
                     ts=None, functions=None,
                     auto_init=False, mode='eval', extra_args=None,
                     active_only=True, verbose=True, kwargs=None):
    """
    Create evaluable object (equations and corresponding variables)
    from the `expression` string.

    Parameters
    ----------
    expression : str
        The expression to evaluate.
    fields : dict
        The dictionary of fields used in `variables`.
    materials : Materials instance
        The materials used in the expression.
    variables : Variables instance
        The variables used in the expression.
    integrals : Integrals instance
        The integrals to be used.
    regions : Region instance or list of Region instances
        The region(s) to be used. If not given, the regions defined
        within the fields domain are used.
    ebcs : Conditions instance, optional
        The essential (Dirichlet) boundary conditions for 'weak'
        mode.
    epbcs : Conditions instance, optional
        The periodic boundary conditions for 'weak'
        mode.
    lcbcs : Conditions instance, optional
        The linear combination boundary conditions for 'weak'
        mode.
    ts : TimeStepper instance, optional
        The time stepper.
    functions : Functions instance, optional
        The user functions for boundary conditions, materials
        etc.
    auto_init : bool
        Set values of all variables to all zeros.
    mode : one of 'eval', 'el_avg', 'qp', 'weak'
        The evaluation mode - 'weak' means the finite element
        assembling, 'qp' requests the values in quadrature points,
        'el_avg' element averages and 'eval' means integration over
        each term region.
    extra_args : dict, optional
        Extra arguments to be passed to terms in the expression.
    active_only : bool
        If True, in 'weak' mode, the (tangent) matrices and residual
        vectors (right-hand sides) contain only active DOFs.
    verbose : bool
        If False, reduce verbosity.
    kwargs : dict, optional
        The variables (dictionary of (variable name) : (Variable
        instance)) to be used in the expression.

    Returns
    -------
    equation : Equation instance
        The equation that is ready to be evaluated.
    variables : Variables instance
        The variables used in the equation.
    """
    if kwargs is None:
        kwargs = {}

    if regions is not None:
        if isinstance(regions, Region):
            regions = [regions]

        regions = OneTypeList(Region, regions)

    else:
        regions = fields[list(fields.keys())[0]].domain.regions

    # Create temporary variables.
    aux_vars = Variables(variables)

    if extra_args is None:
        extra_args = kwargs

    else:
        extra_args = copy(extra_args)
        extra_args.update(kwargs)

    if ts is not None:
        extra_args.update({'ts' : ts})

    equations = Equations.from_conf({'tmp' : expression},
                                    aux_vars, regions, materials, integrals,
                                    user=extra_args, verbose=verbose)
    equations.collect_conn_info()

    # The true variables used in the expression.
    variables = equations.variables
    if auto_init:
        for var in variables:
            var.init_data(step=0)

    if mode == 'weak':
        equations.time_update(ts, ebcs, epbcs, lcbcs, functions,
                              active_only=active_only, verbose=verbose)

    else:
        for eq in equations:
            for term in eq.terms:
                term.time_update(ts)
        setup_extra_data(equations.conn_info)

    return equations, variables


def eval_equations(equations, variables, names=None, preserve_caches=False,
                   mode='eval', dw_mode='vector', term_mode=None,
                   active_only=True, verbose=True):
    """
    Evaluate the equations.

    Parameters
    ----------
    equations : Equations instance
        The equations returned by :func:`create_evaluable()`.
    variables : Variables instance
        The variables returned by :func:`create_evaluable()`.
    names : str or sequence of str, optional
        Evaluate only equations of the given name(s).
    preserve_caches : bool
        If True, do not invalidate evaluate caches of variables.
    mode : one of 'eval', 'el_avg', 'qp', 'weak'
        The evaluation mode - 'weak' means the finite element
        assembling, 'qp' requests the values in quadrature points,
        'el_avg' element averages and 'eval' means integration over
        each term region.
    dw_mode : 'vector' or 'matrix'
        The assembling mode for 'weak' evaluation mode.
    term_mode : str
        The term call mode - some terms support different call modes
        and depending on the call mode different values are
        returned.
    active_only : bool
        If True, in 'weak' mode, the (tangent) matrices and residual
        vectors (right-hand sides) contain only active DOFs.
    verbose : bool
        If False, reduce verbosity.

    Returns
    -------
    out : dict or result
        The evaluation result. In 'weak' mode it is the vector or sparse
        matrix, depending on `dw_mode`. Otherwise, it is a dict of results with
        equation names as keys or a single result for a single equation.
    """
    asm_obj = None

    if mode == 'weak':
        if dw_mode == 'vector':
            asm_obj = equations.create_stripped_state_vector()

        else:
            asm_obj = equations.create_matrix_graph(active_only=active_only,
                                                    verbose=verbose)

    if not preserve_caches:
        equations.invalidate_term_caches()

    out = equations.evaluate(names=names, mode=mode, dw_mode=dw_mode,
                             term_mode=term_mode, asm_obj=asm_obj)

    if variables.has_lcbc and mode == 'weak':
        mtx_lcbc = variables.mtx_lcbc
        if dw_mode == 'vector':
            out = mtx_lcbc.T * out

        elif dw_mode == 'matrix':
            out = mtx_lcbc.T * out * mtx_lcbc
            out = out.tocsr()
            out.sort_indices()

    return out

def eval_in_els_and_qp(expression, iels, coors,
                       fields, materials, variables,
                       functions=None, mode='eval', term_mode=None,
                       extra_args=None, active_only=True, verbose=True,
                       kwargs=None):
    """
    Evaluate an expression in given elements and points.

    Parameters
    ----------
    expression : str
        The expression to evaluate.
    fields : dict
        The dictionary of fields used in `variables`.
    materials : Materials instance
        The materials used in the expression.
    variables : Variables instance
        The variables used in the expression.
    functions : Functions instance, optional
        The user functions for materials etc.
    mode : one of 'eval', 'el_avg', 'qp'
        The evaluation mode - 'qp' requests the values in quadrature points,
        'el_avg' element averages and 'eval' means integration over
        each term region.
    term_mode : str
        The term call mode - some terms support different call modes
        and depending on the call mode different values are
        returned.
    extra_args : dict, optional
        Extra arguments to be passed to terms in the expression.
    active_only : bool
        If True, in 'weak' mode, the (tangent) matrices and residual
        vectors (right-hand sides) contain only active DOFs.
    verbose : bool
        If False, reduce verbosity.
    kwargs : dict, optional
        The variables (dictionary of (variable name) : (Variable
        instance)) to be used in the expression.

    Returns
    -------
    out : array
        The result of the evaluation.
    """
    weights = nm.ones_like(coors[:, 0])
    integral = Integral('ie', coors=coors, weights=weights)

    domain = list(fields.values())[0].domain

    region = Region('Elements', 'given elements', domain, '')
    region.cells = iels
    region.update_shape()
    domain.regions.append(region)

    for field in six.itervalues(fields):
        field.clear_mappings(clear_all=True)
        field.clear_qp_base()

    aux = create_evaluable(expression, fields, materials,
                           variables.itervalues(), Integrals([integral]),
                           functions=functions, mode=mode,
                           extra_args=extra_args, active_only=active_only,
                           verbose=verbose, kwargs=kwargs)
    equations, variables = aux

    out = eval_equations(equations, variables,
                         preserve_caches=False,
                         mode=mode, term_mode=term_mode,
                         active_only=active_only)
    domain.regions.pop()

    return out

def assemble_by_blocks(conf_equations, problem, ebcs=None, epbcs=None,
                       dw_mode='matrix', active_only=True):
    """Instead of a global matrix, return its building blocks as defined in
    `conf_equations`. The name and row/column variables of each block have to
    be encoded in the equation's name, as in::

        conf_equations = {
          'A,v,u' : "dw_lin_elastic.i1.Y2( inclusion.D, v, u )",
        }

    Notes
    -----
    `ebcs`, `epbcs` must be either lists of BC names, or BC configuration
    dictionaries.
    """
    if isinstance( ebcs, list ) and isinstance( epbcs, list ):
        bc_mode = 0
    elif isinstance( ebcs, dict ) and isinstance( epbcs, dict ):
        bc_mode = 1
    else:
        raise TypeError('bad BC!')

    matrices = {}
    for key, mtx_term in six.iteritems(conf_equations):
        ks = key.split( ',' )
        mtx_name, var_names = ks[0], ks[1:]
        output( mtx_name, var_names )

        problem.set_equations({'eq': mtx_term})
        variables = problem.get_variables()
        indx = variables.get_indx

        if bc_mode == 0:
            problem.select_bcs( ebc_names = ebcs, epbc_names = epbcs )

        else:
            problem.time_update(ebcs=ebcs, epbcs=epbcs)

        ir = indx( var_names[0], stripped = True, allow_dual = True )
        ic = indx( var_names[1], stripped = True, allow_dual = True )

        problem.update_materials()
        mtx = problem.evaluate(mtx_term, auto_init=True,
                               mode='weak', dw_mode='matrix',
                               copy_materials=False, active_only=active_only)
        matrices[mtx_name] = mtx[ir,ic]

    return matrices



import numpy as nm

from sfepy.terms.terms import Term, terms
from sfepy.base.base import get_default

def grad_as_vector(grad):
    grad = grad.transpose((0, 1, 3, 2))
    sh = grad.shape
    return grad.reshape((sh[0], sh[1], sh[2] * sh[3], 1))

class AdjDivGradTerm(Term):
    r"""
    Gateaux differential of :math:`\Psi(\ul{u}) = \int_{\Omega} \nu\
    \nabla \ul{v} : \nabla \ul{u}` w.r.t. :math:`\ul{u}` in the direction
    :math:`\ul{v}` or adjoint term to `dw_div_grad`.

    :Definition:

    .. math::
        w \delta_{u} \Psi(\ul{u}) \circ \ul{v}

    :Arguments:
        - material_1 : :math:`w` (weight)
        - material_2 : :math:`\nu` (viscosity)
        - virtual    : :math:`\ul{v}`
        - state      : :math:`\ul{u}`
    """
    name = 'dw_adj_div_grad'
    arg_types = ('material_1', 'material_2', 'virtual', 'parameter')
    arg_shapes = {'material_1' : '1, 1', 'material_2' : '1, 1',
                  'virtual' : ('D', None), 'parameter' : 'D'}

    function = staticmethod(terms.term_ns_asm_div_grad)

    def get_fargs(self, mat1, mat2, virtual, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(state)

        if diff_var is None:
            grad = grad_as_vector(self.get(state, 'grad'))
            fmode = 0

        else:
            grad = nm.array([0], ndmin=4, dtype=nm.float64)
            fmode = 1

        return grad, mat1 * mat2, vg, fmode

class AdjConvect1Term(Term):
    r"""
    The first adjoint term to nonlinear convective term `dw_convect`.

    :Definition:

    .. math::
        \int_{\Omega} ((\ul{v} \cdot \nabla) \ul{u}) \cdot \ul{w}

    :Arguments:
        - virtual   : :math:`\ul{v}`
        - state     : :math:`\ul{w}`
        - parameter : :math:`\ul{u}`
    """
    name = 'dw_adj_convect1'
    arg_types = ('virtual', 'state', 'parameter' )
    arg_shapes = {'virtual' : ('D', 'state'), 'state' : 'D', 'parameter' : 'D'}

    function = staticmethod(terms.dw_adj_convect1)

    def get_fargs(self, virtual, state, parameter,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(state)

        val_w = self.get(state, 'val')
        grad_u = self.get(parameter, 'grad') # No transposition here!

        fmode = diff_var is not None

        return val_w, grad_u, vg, fmode

class AdjConvect2Term(Term):
    r"""
    The second adjoint term to nonlinear convective term `dw_convect`.

    :Definition:

    .. math::
        \int_{\Omega} ((\ul{u} \cdot \nabla) \ul{v}) \cdot \ul{w}

    :Arguments:
        - virtual   : :math:`\ul{v}`
        - state     : :math:`\ul{w}`
        - parameter : :math:`\ul{u}`
    """
    name = 'dw_adj_convect2'
    arg_types = ('virtual', 'state', 'parameter' )
    arg_shapes = {'virtual' : ('D', 'state'), 'state' : 'D', 'parameter' : 'D'}

    function = staticmethod(terms.dw_adj_convect2)

    def get_fargs(self, virtual, state, parameter,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(state)

        val_w = self.get(state, 'val')
        val_u = self.get(parameter, 'val')

        fmode = diff_var is not None

        return val_w, val_u, vg, fmode

class SUPGCAdjStabilizationTerm(Term):
    r"""
    Adjoint term to SUPG stabilization term `dw_st_supg_c`.

    :Definition:

    .. math::
        \sum_{K \in \Ical_h}\int_{T_K} \delta_K\ [ ((\ul{v} \cdot \nabla)
        \ul{u}) ((\ul{u} \cdot \nabla) \ul{w}) + ((\ul{u} \cdot \nabla)
        \ul{u}) ((\ul{v} \cdot \nabla) \ul{w}) ]

    :Arguments:
        - material  : :math:`\delta_K`
        - virtual   : :math:`\ul{v}`
        - state     : :math:`\ul{w}`
        - parameter : :math:`\ul{u}`
    """
    name = 'dw_st_adj_supg_c'
    arg_types = ('material', 'virtual', 'parameter', 'state')
    arg_shapes = {'material' : '1, 1', 'virtual' : ('D', 'state'),
                  'state' : 'D', 'parameter' : 'D'}

    function = staticmethod(terms.dw_st_adj_supg_c)

    def get_fargs(self, mat, virtual, state, parameter,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(state)

        val_u = self.get(parameter, 'val')
        grad_u = self.get(parameter, 'grad').transpose((0, 1, 3, 2)).copy()
        conn = state.field.get_connectivity(self.region, self.integration)

        fmode = diff_var is not None

        return state(), val_u, grad_u, mat, vg, conn, fmode

class SUPGPAdj1StabilizationTerm(Term):
    r"""
    The first adjoint term to SUPG stabilization term `dw_st_supg_p`.

    :Definition:

    .. math::
        \sum_{K \in \Ical_h}\int_{T_K} \delta_K\ \nabla p (\ul{v} \cdot
        \nabla \ul{w})

    :Arguments:
        - material  : :math:`\delta_K`
        - virtual   : :math:`\ul{v}`
        - state     : :math:`\ul{w}`
        - parameter : :math:`p`
    """
    name = 'dw_st_adj1_supg_p'
    arg_types = ('material', 'virtual', 'state', 'parameter')
    arg_shapes = {'material' : '1, 1', 'virtual' : ('D', 'state'),
                  'state' : 'D', 'parameter' : 1}

    function = staticmethod(terms.dw_st_adj1_supg_p)

    def get_fargs(self, mat, virtual, state, parameter,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg_w, _ = self.get_mapping(state)

        grad_p = self.get(parameter, 'grad')
        conn_w = state.field.get_connectivity(self.region, self.integration)

        fmode = diff_var is not None

        return state(), grad_p, mat, vg_w, conn_w, fmode

class SUPGPAdj2StabilizationTerm(Term):
    r"""
    The second adjoint term to SUPG stabilization term `dw_st_supg_p`
    as well as adjoint term to PSPG stabilization term `dw_st_pspg_c`.

    :Definition:

    .. math::
        \sum_{K \in \Ical_h}\int_{T_K} \tau_K\ \nabla r (\ul{v} \cdot \nabla
        \ul{u})

    :Arguments:
        - material  : :math:`\tau_K`
        - virtual   : :math:`\ul{v}`
        - parameter : :math:`\ul{u}`
        - state     : :math:`r`
    """
    name = 'dw_st_adj2_supg_p'
    arg_types = ('material', 'virtual', 'parameter', 'state')
    arg_shapes = {'material' : '1, 1', 'virtual' : ('D', 'state'),
                  'state' : 1, 'parameter' : 'D'}

    function = staticmethod(terms.dw_st_adj2_supg_p)

    def get_fargs(self, mat, virtual, parameter, state,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg_r, _ = self.get_mapping(state)
        vg_u, _ = self.get_mapping(parameter)

        grad_u = self.get(parameter, 'grad').transpose((0, 1, 3, 2)).copy()
        conn_r = state.field.get_connectivity(self.region, self.integration)

        fmode = diff_var is not None

        return grad_u, state(), mat, vg_u, vg_r, conn_r, fmode

class SDDotVolumeTerm(Term):
    r"""
    Sensitivity (shape derivative) of dot product of scalars or vectors.

    :Definition:

    .. math::
        \int_{\Omega} p q (\nabla \cdot \ul{\Vcal}) \mbox{ , }
        \int_{\Omega} (\ul{u} \cdot \ul{w}) (\nabla \cdot \ul{\Vcal})

    :Arguments:
        - parameter_1 : :math:`p` or :math:`\ul{u}`
        - parameter_2 : :math:`q` or :math:`\ul{w}`
        - parameter_mv : :math:`\ul{\Vcal}`
    """
    name = 'd_sd_volume_dot'
    arg_types = ('parameter_1', 'parameter_2', 'parameter_mv')
    arg_shapes = [{'parameter_1' : 'D', 'parameter_2' : 'D',
                   'parameter_mv' : 'D'},
                  {'parameter_1' : 1, 'parameter_2' : 1}]

    function = staticmethod(terms.d_sd_volume_dot)

    def get_fargs(self, par1, par2, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par1)

        val1 = self.get(par1, 'val')
        val2 = self.get(par2, 'val')
        div_mv = self.get(par_mv, 'div')

        return val1, val2, div_mv, vg, get_default(term_mode, 1)

    def get_eval_shape(self, par1, par2, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par1)
        return (n_el, 1, 1, 1), par1.dtype

class SDDivTerm(Term):
    r"""
    Sensitivity (shape derivative) of Stokes term `dw_stokes` in 'div' mode.

    Supports the following term modes: 1 (sensitivity) or 0 (original term
    value).

    :Definition:

    .. math::
        \int_{\Omega} p [ (\nabla \cdot \ul{w}) (\nabla \cdot \ul{\Vcal})
        - \pdiff{\Vcal_k}{x_i} \pdiff{w_i}{x_k} ]

    :Arguments:
        - parameter_u : :math:`\ul{u}`
        - parameter_p : :math:`p`
        - parameter_mv : :math:`\ul{\Vcal}`
    """
    name = 'd_sd_div'
    arg_types = ('parameter_u', 'parameter_p', 'parameter_mv')
    arg_shapes = {'parameter_u' : 'D', 'parameter_p' : 1,
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_div)

    def get_fargs(self, par_u, par_p, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_u)

        div_u = self.get(par_u, 'div')
        grad_u = grad_as_vector(self.get(par_u, 'grad'))
        val_p = self.get(par_p, 'val')
        div_mv = self.get(par_mv, 'div')
        grad_mv = grad_as_vector(self.get(par_mv, 'grad'))

        return (div_u, grad_u, val_p, div_mv, grad_mv, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, par_u, par_p, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_u)
        return (n_el, 1, 1, 1), par_u.dtype

class SDDivGradTerm(Term):
    r"""
    Sensitivity (shape derivative) of diffusion term `dw_div_grad`.

    Supports the following term modes: 1 (sensitivity) or 0 (original term
    value).

    :Definition:

    .. math::
        w \nu \int_{\Omega} [ \pdiff{u_i}{x_k} \pdiff{w_i}{x_k}
        (\nabla \cdot \ul{\Vcal})
        - \pdiff{\Vcal_j}{x_k} \pdiff{u_i}{x_j} \pdiff{w_i}{x_k}
        - \pdiff{u_i}{x_k} \pdiff{\Vcal_l}{x_k} \pdiff{w_i}{x_k} ]

    :Arguments:
        - material_1  : :math:`w` (weight)
        - material_2  : :math:`\nu` (viscosity)
        - parameter_u : :math:`\ul{u}`
        - parameter_w : :math:`\ul{w}`
        - parameter_mv : :math:`\ul{\Vcal}`
    """
    name = 'd_sd_div_grad'
    arg_types = ('material_1', 'material_2', 'parameter_u', 'parameter_w',
                 'parameter_mv')
    arg_shapes = {'material_1' : '1, 1', 'material_2' : '1, 1',
                  'parameter_u' : 'D', 'parameter_w' : 'D',
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_div_grad)

    def get_fargs(self, mat1, mat2, par_u, par_w, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_u)

        grad_u = grad_as_vector(self.get(par_u, 'grad'))
        grad_w = grad_as_vector(self.get(par_w, 'grad'))
        div_mv = self.get(par_mv, 'div')
        grad_mv = grad_as_vector(self.get(par_mv, 'grad'))

        return (grad_u, grad_w, div_mv, grad_mv, mat1 * mat2, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, mat1, mat2, par_u, par_w, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_u)
        return (n_el, 1, 1, 1), par_u.dtype

class SDConvectTerm(Term):
    r"""
    Sensitivity (shape derivative) of convective term `dw_convect`.

    Supports the following term modes: 1 (sensitivity) or 0 (original term
    value).

    :Definition:

    .. math::
        \int_{\Omega} [ u_k \pdiff{u_i}{x_k} w_i (\nabla \cdot \Vcal)
        - u_k \pdiff{\Vcal_j}{x_k} \pdiff{u_i}{x_j} w_i ]

    :Arguments:
        - parameter_u : :math:`\ul{u}`
        - parameter_w : :math:`\ul{w}`
        - parameter_mv : :math:`\ul{\Vcal}`
    """
    name = 'd_sd_convect'
    arg_types = ('parameter_u', 'parameter_w', 'parameter_mv')
    arg_shapes = {'parameter_u' : 'D', 'parameter_w' : 'D',
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_convect)

    def get_fargs(self, par_u, par_w, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_u)

        val_u = self.get(par_u, 'val')
        grad_u = grad_as_vector(self.get(par_u, 'grad'))
        val_w = self.get(par_w, 'val')
        div_mv = self.get(par_mv, 'div')
        grad_mv = grad_as_vector(self.get(par_mv, 'grad'))

        return (val_u, grad_u, val_w, div_mv, grad_mv, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, par_u, par_w, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_u)
        return (n_el, 1, 1, 1), par_u.dtype

class NSOFMinGradTerm(Term):
    name = 'd_of_ns_min_grad'
    arg_types = ('material_1', 'material_2', 'parameter')
    arg_shapes = {'material_1' : '1, 1', 'material_2' : '1, 1',
                  'parameter' : 1}

    function = staticmethod(terms.d_of_nsMinGrad)

    def get_fargs(self, weight, mat, parameter,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(parameter)

        grad = grad_as_vector(self.get(parameter, 'grad'))

        return grad, weight * mat, vg

    def get_eval_shape(self, weight, mat, parameter,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        return (1, 1, 1, 1), parameter.dtype

class NSOFSurfMinDPressTerm(Term):
    r"""
    Sensitivity of :math:`\Psi(p)`.

    :Definition:

    .. math::
        \delta \Psi(p) = \delta \left( \int_{\Gamma_{in}}p -
        \int_{\Gamma_{out}}bpress \right)

    :Arguments:
        - material_1 : :math:`w` (weight)
        - material_2 : :math:`bpress` (given pressure)
        - parameter  : :math:`p`
    """
    name = 'd_of_ns_surf_min_d_press'
    arg_types = ('material_1', 'material_2', 'parameter')
    arg_shapes = {'material_1' : 1, 'material_2' : 1,
                  'parameter' : 1}
    integration = 'surface'

    function = staticmethod(terms.d_of_nsSurfMinDPress)

    def get_fargs(self, weight, bpress, parameter,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        sg, _ = self.get_mapping(parameter)

        val_p = self.get(parameter, 'val')

        return val_p, weight, bpress, sg, 0

    def get_eval_shape(self, weight, bpress, parameter,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        return (1, 1, 1, 1), parameter.dtype

class NSOFSurfMinDPressDiffTerm(NSOFSurfMinDPressTerm):
    r"""
    Gateaux differential of :math:`\Psi(p)` w.r.t. :math:`p` in the
    direction :math:`q`.

    :Definition:

    .. math::
        w \delta_{p} \Psi(p) \circ q

    :Arguments:
        - material : :math:`w` (weight)
        - virtual  : :math:`q`
    """
    name = 'dw_of_ns_surf_min_d_press_diff'
    arg_types = ('material', 'virtual')
    arg_shapes = {'material' : 1, 'virtual' : (1, None)}

    def get_fargs(self, weight, virtual,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        sg, _ = self.get_mapping(virtual)

        aux = nm.array([0], ndmin=4, dtype=nm.float64)

        return aux, weight, 0.0, sg, 1

class SDGradDivStabilizationTerm(Term):
    r"""
    Sensitivity (shape derivative) of stabilization term `dw_st_grad_div`.

    :Definition:

    .. math::
        \gamma \int_{\Omega} [ (\nabla \cdot \ul{u}) (\nabla \cdot \ul{w})
        (\nabla \cdot \ul{\Vcal})
        - \pdiff{u_i}{x_k} \pdiff{\Vcal_k}{x_i} (\nabla \cdot \ul{w})
        - (\nabla \cdot \ul{u}) \pdiff{w_i}{x_k} \pdiff{\Vcal_k}{x_i} ]

    :Arguments:
        - material    : :math:`\gamma`
        - parameter_u : :math:`\ul{u}`
        - parameter_w : :math:`\ul{w}`
        - parameter_mv : :math:`\ul{\Vcal}`
        - mode        : 1 (sensitivity) or 0 (original term value)
    """
    name = 'd_sd_st_grad_div'
    arg_types = ('material', 'parameter_u', 'parameter_w',
                 'parameter_mv')
    arg_shapes = {'material' : '1, 1',
                  'parameter_u' : 'D', 'parameter_w' : 'D',
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_st_grad_div)

    def get_fargs(self, mat, par_u, par_w, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_u)

        div_u = self.get(par_u, 'div')
        grad_u = grad_as_vector(self.get(par_u, 'grad'))
        div_w = self.get(par_w, 'div')
        grad_w = grad_as_vector(self.get(par_w, 'grad'))
        div_mv = self.get(par_mv, 'div')
        grad_mv = grad_as_vector(self.get(par_mv, 'grad'))

        return (div_u, grad_u, div_w, grad_w, div_mv, grad_mv, mat, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, mat, par_u, par_w, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_u)
        return (n_el, 1, 1, 1), par_u.dtype

class SDSUPGCStabilizationTerm(Term):
    r"""
    Sensitivity (shape derivative) of stabilization term `dw_st_supg_c`.

    :Definition:

    .. math::
        \sum_{K \in \Ical_h}\int_{T_K} \delta_K\ [ (\ul{b} \cdot \nabla u_k)
        (\ul{b} \cdot \nabla w_k) (\nabla \cdot \Vcal) -
        (\ul{b} \cdot \nabla \Vcal_i) \pdiff{u_k}{x_i}
        (\ul{b} \cdot \nabla w_k) - (\ul{u} \cdot \nabla u_k)
        (\ul{b} \cdot \nabla \Vcal_i) \pdiff{w_k}{x_i} ]

    :Arguments:
        - material    : :math:`\delta_K`
        - parameter_b : :math:`\ul{b}`
        - parameter_u : :math:`\ul{u}`
        - parameter_w : :math:`\ul{w}`
        - parameter_mv : :math:`\ul{\Vcal}`
        - mode        : 1 (sensitivity) or 0 (original term value)
    """
    name = 'd_sd_st_supg_c'
    arg_types = ('material', 'parameter_b', 'parameter_u', 'parameter_w',
                'parameter_mv')
    arg_shapes = {'material' : '1, 1',
                  'parameter_b' : 'D', 'parameter_u' : 'D', 'parameter_w' : 'D',
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_st_supg_c)

    def get_fargs(self, mat, par_b, par_u, par_w, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_u)

        val_b = self.get(par_b, 'val')
        grad_u = self.get(par_u, 'grad').transpose((0, 1, 3, 2)).copy()
        grad_w = self.get(par_w, 'grad').transpose((0, 1, 3, 2)).copy()
        div_mv = self.get(par_mv, 'div')
        grad_mv = self.get(par_mv, 'grad').transpose((0, 1, 3, 2)).copy()

        return (val_b, grad_u, grad_w, div_mv, grad_mv, mat, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, mat, par_b, par_u, par_w, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_u)
        return (n_el, 1, 1, 1), par_u.dtype

class SDPSPGCStabilizationTerm(Term):
    r"""
    Sensitivity (shape derivative) of stabilization terms `dw_st_supg_p` or
    `dw_st_pspg_c`.

    :Definition:

    .. math::
        \sum_{K \in \Ical_h}\int_{T_K} \delta_K\
        [ \pdiff{r}{x_i} (\ul{b} \cdot \nabla u_i) (\nabla \cdot \Vcal) -
        \pdiff{r}{x_k} \pdiff{\Vcal_k}{x_i} (\ul{b} \cdot \nabla u_i)
        - \pdiff{r}{x_k} (\ul{b} \cdot \nabla \Vcal_k) \pdiff{u_i}{x_k} ]

    :Arguments:
        - material    : :math:`\delta_K`
        - parameter_b : :math:`\ul{b}`
        - parameter_u : :math:`\ul{u}`
        - parameter_r : :math:`r`
        - parameter_mv : :math:`\ul{\Vcal}`
        - mode        : 1 (sensitivity) or 0 (original term value)
    """
    name = 'd_sd_st_pspg_c'
    arg_types = ('material', 'parameter_b', 'parameter_u', 'parameter_r',
                'parameter_mv')
    arg_shapes = {'material' : '1, 1',
                  'parameter_b' : 'D', 'parameter_u' : 'D', 'parameter_r' : 1,
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_st_pspg_c)

    def get_fargs(self, mat, par_b, par_u, par_r, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_u)

        val_b = self.get(par_b, 'val')
        grad_u = self.get(par_u, 'grad').transpose((0, 1, 3, 2)).copy()
        grad_r = self.get(par_r, 'grad')
        div_mv = self.get(par_mv, 'div')
        grad_mv = self.get(par_mv, 'grad').transpose((0, 1, 3, 2)).copy()

        return (val_b, grad_u, grad_r, div_mv, grad_mv, mat, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, mat, par_b, par_u, par_r, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_u)
        return (n_el, 1, 1, 1), par_u.dtype

class SDPSPGPStabilizationTerm(Term):
    r"""
    Sensitivity (shape derivative) of stabilization term `dw_st_pspg_p`.

    :Definition:

    .. math::
        \sum_{K \in \Ical_h}\int_{T_K} \tau_K\ [ (\nabla r \cdot \nabla p)
        (\nabla \cdot \Vcal) - \pdiff{r}{x_k} (\nabla \Vcal_k \cdot \nabla p) -
        (\nabla r \cdot \nabla \Vcal_k) \pdiff{p}{x_k} ]

    :Arguments:
        - material    : :math:`\tau_K`
        - parameter_r : :math:`r`
        - parameter_p : :math:`p`
        - parameter_mv : :math:`\ul{\Vcal}`
        - mode        : 1 (sensitivity) or 0 (original term value)
    """
    name = 'd_sd_st_pspg_p'
    arg_types = ('material', 'parameter_r', 'parameter_p',
                'parameter_mv')
    arg_shapes = {'material' : '1, 1',
                  'parameter_r' : 1, 'parameter_p' : 1,
                  'parameter_mv' : 'D'}

    function = staticmethod(terms.d_sd_st_pspg_p)

    def get_fargs(self, mat, par_r, par_p, par_mv,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        vg, _ = self.get_mapping(par_p)

        grad_r = self.get(par_r, 'grad')
        grad_p = self.get(par_p, 'grad')
        div_mv = self.get(par_mv, 'div')
        grad_mv = self.get(par_mv, 'grad').transpose((0, 1, 3, 2)).copy()

        return (grad_r, grad_p, div_mv, grad_mv, mat, vg,
                get_default(term_mode, 1))

    def get_eval_shape(self, mat, par_r, par_p, par_mv,
                       mode=None, term_mode=None, diff_var=None, **kwargs):
        n_el, n_qp, dim, n_en, n_c = self.get_data_shape(par_p)
        return (n_el, 1, 1, 1), par_p.dtype



import numpy as nm

from sfepy.base.base import Struct
from sfepy.terms.terms import Term

class THTerm(Term):
    """
    Base class for terms depending on time history (fading memory
    terms).
    """

    def eval_real(self, shape, fargs, mode='eval', term_mode=None,
                  diff_var=None, **kwargs):

        if diff_var is None:
            if mode == 'eval':
                out = 0.0

            else:
                out = nm.zeros(shape, dtype=nm.float64)

            iter_kernel = fargs
            for ii, fargs in iter_kernel():
                out1, status = Term.eval_real(self, shape, fargs, mode=mode,
                                              term_mode=term_mode,
                                              diff_var=diff_var, **kwargs)
                out += out1

        else:
            out, status = Term.eval_real(self, shape, fargs, mode=mode,
                                         term_mode=term_mode,
                                         diff_var=diff_var, **kwargs)

        return out, status

class ETHTerm(Term):
    """
    Base class for terms depending on time history with exponential
    convolution kernel (fading memory terms).
    """

    def get_eth_data(self, key, state, decay, values):
        step_cache = state.evaluate_cache.setdefault('eth', {})
        cache = step_cache.setdefault(None, {})

        data_key = key + (self.arg_derivatives[state.name],)
        if data_key in cache:
            out = cache[data_key]
            out.values = values

        else:
            out = Struct(history=nm.zeros_like(values),
                         values=values,
                         decay=decay,
                         __advance__=self.advance_eth_data)
            cache[data_key] = out

        return out

    def advance_eth_data(self, ts, data):
        data.history[:] = data.decay * (data.history + data.values)



"""
Grammar for selecting regions of a domain.

Regions serve for selection of certain parts of the computational domain
represented as a finite element mesh. They are used to define the boundary
conditions, the domains of terms and materials etc.

Notes
-----
History: pre-git versions already from from 13.06.2006.
"""
from __future__ import print_function
from pyparsing import Literal, CaselessLiteral, Word, delimitedList,\
     Group, Optional, ZeroOrMore, nums, alphas, alphanums,\
     Combine, StringStart, StringEnd, Forward, oneOf, ParseException

ParseException # Needed for importing elsewhere.

op_codes = ['OA_SubV', 'OA_SubE', 'OA_SubF', 'OA_SubC', 'OA_SubS',
            'OA_AddV', 'OA_AddE', 'OA_AddF', 'OA_AddC', 'OA_AddS',
            'OA_IntersectV', 'OA_IntersectE', 'OA_IntersectF',
            'OA_IntersectC', 'OA_IntersectS']
eval_codes = ['E_VIR', 'E_VOS', 'E_VBF', 'E_VOG', 'E_OVIR', 'E_VI', 'E_VOSET',
              'E_CBF', 'E_COG', 'E_CI', 'E_COSET']
kw_codes = ['KW_All', 'KW_Region']

def to_stack(stack):
    def push_first(str, loc, toks):
        if toks:
            stack.append(toks[0])
        return toks
    return push_first

def replace(what, keep=False):
    def _replace(str, loc, toks):
        ret = {'token' : what, 'orig' : []}
        if keep:
            ret['orig'] = list(toks[0])
        return ret
    return _replace

def replace_with_region(what, r_index):
    def _replace(str, loc, toks):
        ret = {'token' : what, 'orig' : []}

        orig = toks[0]
        r_orig = orig[r_index]
        if isinstance(r_orig, dict) and (r_orig['token'] == 'KW_Region'):
            orig = list(orig[:r_index]) + r_orig['orig']
        ret['orig'] = orig
        return ret
    return _replace

def join_tokens(str, loc, toks):
    return [" ".join(toks[0])]

def visit_stack(stack, op_visitor, leaf_visitor):

    def visit(stack, level):
        op = stack.pop()

        token = op['token']
        if token in op_codes:
            res2 = visit(stack, level + 1)
            res1 = visit(stack, level + 1)
            return op_visitor(level, op, res1, res2)

        elif token in eval_codes:
            return leaf_visitor(level, op)

        elif token in kw_codes:
            return leaf_visitor(level, op)

        else:
            raise ValueError(token)

    return visit(stack, 0)

def print_op(level, op, item1, item2):
    print(level * '  ' + (': %s' % op))

def print_leaf(level, op):
    print(level * '  ' + ('< %s' % op))

def print_stack(stack):
    visit_stack(stack, print_op, print_leaf)

def create_bnf(stack):
    point = Literal(".")
    e = CaselessLiteral("E")
    inumber = Word(nums)
    fnumber = Combine(Word("+-"+nums, nums) +
                       Optional(point + Optional(Word(nums))) +
                       Optional(e + Word("+-"+nums, nums)))
    _of = Literal('of')
    _in = Literal('in')
    _by = Literal('by')
    _copy = Literal('copy')

    _mv = Literal('-v').setParseAction(replace('OA_SubV'))
    _me = Literal('-e').setParseAction(replace('OA_SubE'))
    _mf = Literal('-f').setParseAction(replace('OA_SubF'))
    _mc = Literal('-c').setParseAction(replace('OA_SubC'))
    _ms = Literal('-s').setParseAction(replace('OA_SubS'))
    _pv = Literal('+v').setParseAction(replace('OA_AddV'))
    _pe = Literal('+e').setParseAction(replace('OA_AddE'))
    _pf = Literal('+f').setParseAction(replace('OA_AddF'))
    _pc = Literal('+c').setParseAction(replace('OA_AddC'))
    _ps = Literal('+s').setParseAction(replace('OA_AddS'))
    _inv = Literal('*v').setParseAction(replace('OA_IntersectV'))
    _ine = Literal('*e').setParseAction(replace('OA_IntersectE'))
    _inf = Literal('*f').setParseAction(replace('OA_IntersectF'))
    _inc = Literal('*c').setParseAction(replace('OA_IntersectC'))
    _ins = Literal('*s').setParseAction(replace('OA_IntersectS'))
    regop = (_mv | _me | _mf | _mc | _ms |
             _pv | _pe | _pf | _pc | _ps |
             _inv | _ine | _inf | _inc | _ins)

    lpar  = Literal("(").suppress()
    rpar  = Literal(")").suppress()

    _all = Literal('all').setParseAction(replace('KW_All'))
    vertex = Literal('vertex')
    vertices = Literal('vertices')
    cell = Literal('cell')
    cells = Literal('cells')
    group = Literal('group')
    _set = Literal('set')
    surface = Literal('surface')

    ident = Word(alphas + '_.', alphanums + '_.')
    set_name = Word(nums) | ident

    function = Word(alphas + '_', alphanums + '_')
    function = Group(function).setParseAction(join_tokens)

    region = Combine(Literal('r.') + Word(alphas + '_-',
                                          '_-' + alphas + nums + '.'))
    region = Group(Optional(_copy, default='nocopy') + region)
    region.setParseAction(replace('KW_Region', keep=True))

    coor = oneOf('x y z')
    boolop = oneOf('& |')
    relop = oneOf('< > <= >= != ==')
    bool_term = (ZeroOrMore('(') + (coor | fnumber) + relop + (coor | fnumber)
                 + ZeroOrMore(')'))
    relation = Forward()
    relation << (ZeroOrMore('(')
                 + bool_term + ZeroOrMore(boolop + relation)
                 + ZeroOrMore(')'))
    relation = Group(relation).setParseAction(join_tokens)

    nos = Group(vertices + _of + surface).setParseAction(replace('E_VOS'))
    nir = Group(vertices + _in + relation).setParseAction(
        replace('E_VIR', keep=True))
    nbf = Group(vertices + _by + function).setParseAction(
        replace('E_VBF', keep=True))
    ebf = Group(cells + _by + function).setParseAction(
        replace('E_CBF', keep=True))
    eog = Group(cells + _of + group + Word(nums)).setParseAction(
        replace('E_COG', keep=True))
    nog = Group(vertices + _of + group + Word(nums)).setParseAction(
        replace('E_VOG', keep=True))
    onir = Group(vertex + _in + region).setParseAction(
        replace_with_region('E_OVIR', 2))
    ni = Group(vertex + delimitedList(inumber)).setParseAction(
        replace('E_VI', keep=True))
    ei = Group(cell + delimitedList(inumber)).setParseAction(
        replace('E_CI', keep=True))
    noset = Group(vertices + _of + _set + set_name).setParseAction(
        replace('E_VOSET', keep=True))
    eoset = Group(cells + _of + _set + set_name).setParseAction(
        replace('E_COSET', keep=True))

    region_expression = Forward()

    atom1 = (_all | region | ni | onir | nos | nir | nbf
             | ei | ebf | eog | nog | noset | eoset)
    atom1.setParseAction(to_stack(stack))
    atom2 = (lpar + region_expression.suppress() + rpar)
    atom = (atom1 | atom2)

    aux = (regop + region_expression)
    aux.setParseAction(to_stack(stack))
    region_expression << atom + ZeroOrMore(aux)
    region_expression = StringStart() + region_expression + StringEnd()

    return region_expression



r"""
Laplace equation using the short syntax of keywords.

See :ref:`diffusion-poisson` for the long syntax version.

Find :math:`t` such that:

.. math::
    \int_{\Omega} c \nabla s \cdot \nabla t
    = 0
    \;, \quad \forall s \;.
"""
from __future__ import absolute_import
from sfepy import data_dir

filename_mesh = data_dir + '/meshes/3d/cylinder.mesh'

materials = {
    'coef' : ({'val' : 1.0},),
}

regions = {
    'Omega' : 'all', # or 'cells of group 6'
    'Gamma_Left' : ('vertices in (x < 0.00001)', 'facet'),
    'Gamma_Right' : ('vertices in (x > 0.099999)', 'facet'),
}

fields = {
    'temperature' : ('real', 1, 'Omega', 1),
}

variables = {
    't' : ('unknown field', 'temperature', 0),
    's' : ('test field',    'temperature', 't'),
}

ebcs = {
    't1' : ('Gamma_Left', {'t.0' : 2.0}),
    't2' : ('Gamma_Right', {'t.0' : -2.0}),
}

integrals = {
    'i' : 2,
}

equations = {
    'Temperature' : """dw_laplace.i.Omega( coef.val, s, t ) = 0"""
}

solvers = {
    'ls' : ('ls.scipy_direct', {}),
    'newton' : ('nls.newton',
                {'i_max'      : 1,
                 'eps_a'      : 1e-10,
    }),
}

options = {
    'nls' : 'newton',
    'ls' : 'ls',
}



#!/usr/bin/env python
"""
See the :ref:`sec-mat_optim` tutorial for a comprehensive description of this
example.
"""
from __future__ import print_function
from __future__ import absolute_import
import sys
sys.path.append('.')

import numpy as nm
from scipy.optimize import fmin_tnc

from sfepy.base.base import Struct
from sfepy.base.log import Log

class MaterialOptimizer(object):

    @staticmethod
    def create_app(filename, is_homog=False, **kwargs):
        from sfepy.base.conf import ProblemConf, get_standard_keywords
        from sfepy.homogenization.homogen_app import HomogenizationApp
        from sfepy.applications import PDESolverApp

        required, other = get_standard_keywords()
        if is_homog:
            required.remove('equations')

        conf = ProblemConf.from_file(filename, required, other,
                                     define_args=kwargs)
        options = Struct(output_filename_trunk=None,
                         save_ebc=False,
                         save_ebc_nodes=False,
                         save_regions=False,
                         save_regions_as_groups=False,
                         save_field_meshes=False,
                         solve_not=False)

        if is_homog:
            app = HomogenizationApp(conf, options, 'material_opt_micro:')

        else:
            app = PDESolverApp(conf, options, 'material_opt_macro:')

        app.conf.opt_data = {}
        opts = conf.options
        if hasattr(opts, 'parametric_hook'):  # Parametric study.
            parametric_hook = conf.get_function(opts.parametric_hook)
            app.parametrize(parametric_hook)

        return app

    def x_norm2real(self, x):
        return x * (self.x_U - self.x_L) + self.x_L

    def x_real2norm(self, x):
        return (x - self.x_L) / (self.x_U - self.x_L)

    def __init__(self, macro_fn, micro_fn, x0, x_L, x_U, exp_data):
        self.macro_app = self.create_app(macro_fn, is_homog=False, is_opt=True)
        self.micro_app = self.create_app(micro_fn, is_homog=True, is_opt=True)
        self.x_L = nm.array(x_L)
        self.x_U = nm.array(x_U)
        self.x0 = self.x_real2norm(nm.array(x0))
        self.x = []
        self.eval_f = []
        self.exp_data = exp_data

    @staticmethod
    def rotate_mat(D, angle):
        s = nm.sin(angle)
        c = nm.cos(angle)
        s2 = s**2
        c2 = c**2
        sc = s * c
        T = nm.array([[c2, 0, s2, 0, 2*sc,0],
                      [0, 1, 0, 0, 0, 0],
                      [s2, 0, c2, 0, -2*sc, 0],
                      [0, 0, 0, c, 0, -s],
                      [-sc, 0, sc, 0, c2 - s2, 0],
                      [0, 0, 0, s, 0, c]])

        return nm.dot(nm.dot(T, D), T.T)

    def matopt_eval(self, x):
        mic_od = self.micro_app.conf.opt_data
        mac_od = self.macro_app.conf.opt_data

        mic_od['coefs'] = {}
        mic_od['mat_params'] = x
        self.micro_app()

        D = mic_od['D_homog']
        val = 0.0
        aux = []
        for phi, exp_k in self.exp_data:
            print('phi = %d' % phi)

            mac_od['D_homog'] = self.rotate_mat(D, nm.deg2rad(phi))
            self.macro_app()

            comp_k = mac_od['k']
            val += (1.0 - comp_k / exp_k)**2
            aux.append((comp_k, exp_k))

        val = nm.sqrt(val)
        self.x.append(x)
        self.eval_f.append(val)

        return val

    def iter_step(self, x, first_step=False):
        if first_step:
            self.log = Log([['O'], ['E_f', 'E_m'], ['v_f', 'v_m']],
                ylabels=['Obj. fun.', "Young's modulus", "Poisson's ratio"],
                xlabels=['iter', 'iter', 'iter'],
                aggregate=0)
            self.istep = 0
            self.log(0.5, x[0], x[2], x[1], x[3],
                x=[0, 0, 0, 0])
        else:
            self.istep += 1
            self.log(self.eval_f[-1], x[0], x[2], x[1], x[3],
                x=(self.istep,)*4)

    def material_optimize(self):
        x0 = self.x0
        bnds = zip(self.x_real2norm(self.x_L), self.x_real2norm(self.x_U))
        feval = lambda x: self.matopt_eval(self.x_norm2real(x))
        istep = lambda x: self.iter_step(self.x_norm2real(x))
        self.iter_step(self.x_norm2real(x0), first_step=True)

        print('>>> material optimization START <<<')
        xopt = fmin_tnc(feval, x0, approx_grad=True, bounds=list(bnds),
                        xtol=1e-3, callback=istep)
        print('>>> material optimization FINISHED <<<')

        self.log(finished=True)
        return self.x_norm2real(xopt[0])

def main():
    srcdir = 'examples/homogenization/'
    micro_filename = srcdir + 'homogenization_opt.py'
    macro_filename = srcdir + 'linear_elasticity_opt.py'

    exp_data = zip([0, 30, 60, 90], [1051140., 197330., 101226., 95474.])
    mo = MaterialOptimizer(macro_filename, micro_filename,
                           [160.e9, 0.25, 5.e9, 0.45],
                           [120e9, 0.2, 2e9, 0.2],
                           [200e9, 0.45, 8e9, 0.45],
                           list(exp_data))

    optim_par = mo.material_optimize()
    print('optimized parameters: ', optim_par)

if __name__ == '__main__':
    main()



from __future__ import absolute_import
import numpy as nm

from sfepy.base.base import assert_, get_default, Struct
from sfepy.homogenization.coefs_base import CorrMiniApp, CoefN
import six
from six.moves import range

class CorrRegion( CorrMiniApp ):

    def __init__( self, name, problem, kwargs ):
        CorrMiniApp.__init__( self, name, problem, kwargs )
        self.set_default('Nreg', len(list(self.regions.values())[0]))
        self.set_default('ebcs_list', False)

    def get_variables( self, ir, data ):
        return iter([])
        
    def __call__( self, problem = None, data = None ):
        problem = get_default( problem, self.problem )

        states = nm.zeros( (self.Nreg,), dtype = nm.object )
        clist = []
        for ir in range( self.Nreg ):

            problem.select_variables( self.variables )
            
            for name, val in self.get_variables( ir, data ):
                problem.variables[name].set_data(val)

            equations = {}
        
            for keye, vale in six.iteritems(self.equations):
                for keyr, valr in six.iteritems(self.regions):
                    vale = vale.replace( keyr, valr[ir] )
            
                equations[keye] = vale    

            problem.set_equations( equations )

            if ( self.ebcs_list ):
                problem.select_bcs( ebc_names = self.ebcs[ir], epbc_names = self.epbcs )
            else:
                problem.select_bcs( ebc_names = self.ebcs, epbc_names = self.epbcs )

            self.init_solvers(problem)

            state = problem.solve()
            assert_(state.has_ebc())
            states[ir] = state()
            clist.append( (ir,) )

        self.save( states, problem, clist )

        return Struct( name = self.name,
                       states = states,
                       di = problem.variables.di )

class CoefRegion( CoefN ):

    def __init__( self, name, problem, kwargs ):
        CoefN.__init__( self, name, problem, kwargs )
        self.corr_dim = len(list(kwargs['regions'].values())[0])
        
    def get_variables( self, problem, ir, data ):

        corr = data[self.requires[-1]]
        yield (self.variables[0], corr.states[ir])




from __future__ import absolute_import
import numpy as nm

from sfepy.terms.terms import Term
from sfepy.linalg import dot_sequences

class NonPenetrationTerm(Term):
    r"""
    Non-penetration condition in the weak sense.

    :Definition:

    .. math::
        \int_{\Gamma} c \lambda \ul{n} \cdot \ul{v} \mbox{ , }
        \int_{\Gamma} c \hat\lambda \ul{n} \cdot \ul{u} \\
        \int_{\Gamma} \lambda \ul{n} \cdot \ul{v} \mbox{ , }
        \int_{\Gamma} \hat\lambda \ul{n} \cdot \ul{u}

    :Arguments 1:
        - material : :math:`c` (optional)
        - virtual  : :math:`\ul{v}`
        - state    : :math:`\lambda`

    :Arguments 2:
        - material : :math:`c` (optional)
        - state    : :math:`\ul{u}`
        - virtual  : :math:`\hat\lambda`
    """
    name = 'dw_non_penetration'
    arg_types = (('opt_material', 'virtual', 'state'),
                 ('opt_material', 'state', 'virtual'))
    arg_shapes = [{'opt_material' : '1, 1',
                   'virtual/grad' : ('D', None), 'state/grad' : 1,
                   'virtual/div' : (1, None), 'state/div' : 'D'},
                  {'opt_material' : None}]
    modes = ('grad', 'div')
    integration = 'surface'

    @staticmethod
    def function(out, val_qp, ebf, bf, mat, sg, diff_var, mode):
        """
        `ebf` belongs to vector variable, `bf` to scalar variable.
        """
        normals = sg.normal
        n_fa = out.shape[0]

        if diff_var is None:
            if mode == 'grad':
                ebf_t = nm.tile(ebf.transpose((0, 1, 3, 2)), (n_fa, 1, 1, 1))

                nl = normals * val_qp
                eftnl = mat * dot_sequences(ebf_t, nl)
                status = sg.integrate(out, eftnl, 0)

            else:
                bf_t = nm.tile(bf.transpose((0, 1, 3, 2)), (n_fa, 1, 1, 1))

                ntu = (normals * val_qp).sum(axis=-2)[...,None]
                ftntu = mat * (bf_t * ntu)

                status = sg.integrate(out, ftntu, 0)

        else:
            ebf_t = nm.tile(ebf.transpose((0, 1, 3, 2)), (n_fa, 1, 1, 1))
            bf_ = nm.tile(bf, (n_fa, 1, 1, 1))

            eftn = mat * dot_sequences(ebf_t, normals)
            eftnf = eftn * bf_

            if mode == 'grad':
                status = sg.integrate(out, eftnf, 0)

            else:
                ftntef = nm.ascontiguousarray(eftnf.transpose((0, 1, 3, 2)))
                status = sg.integrate(out, ftntef, 0)

        return status

    def get_fargs(self, mat, vvar, svar,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        from sfepy.discrete.variables import expand_basis

        if self.mode == 'grad':
            qp_var = svar

        else:
            qp_var = vvar

        val_qp = self.get(qp_var, 'val')

        vsg, _ = self.get_mapping(vvar)
        ssg, _ = self.get_mapping(svar)
        n_fa, n_qp, dim, n_fn, n_c = self.get_data_shape(vvar)

        if mat is None:
            mat = nm.ones((1, n_qp, 1, 1), dtype=nm.float64)

        ebf = expand_basis(vsg.bf, dim)

        return val_qp, ebf, ssg.bf, mat, vsg, diff_var, self.mode

class NonPenetrationPenaltyTerm(Term):
    r"""
    Non-penetration condition in the weak sense using a penalty.

    :Definition:

    .. math::
        \int_{\Gamma} c (\ul{n} \cdot \ul{v}) (\ul{n} \cdot \ul{u})

    :Arguments:
        - material : :math:`c`
        - virtual  : :math:`\ul{v}`
        - state    : :math:`\ul{u}`
    """
    name = 'dw_non_penetration_p'
    arg_types = ('material', 'virtual', 'state')
    arg_shapes = {'material' : '1, 1',
                  'virtual' : ('D', 'state'), 'state' : 'D'}
    integration = 'surface'

    @staticmethod
    def function(out, val_qp, ebf, mat, sg, diff_var):
        normals = sg.normal
        n_fa = out.shape[0]

        ebf_t = nm.tile(ebf.transpose((0, 1, 3, 2)), (n_fa, 1, 1, 1))

        if diff_var is None:
            nu = dot_sequences(normals, val_qp, 'ATB')
            nt = dot_sequences(ebf_t, normals)
            entnu = mat * nt * nu
            status = sg.integrate(out, entnu, 0)

        else:
            nt = dot_sequences(ebf_t, normals)
            entn = mat * dot_sequences(nt, nt, 'ABT')
            status = sg.integrate(out, entn, 0)

        return status

    def get_fargs(self, mat, vvar, svar,
                  mode=None, term_mode=None, diff_var=None, **kwargs):
        from sfepy.discrete.variables import expand_basis

        if diff_var is None:
            val_qp = self.get(svar, 'val')

        else:
            val_qp = nm.array([0], ndmin=4, dtype=nm.float64)

        sg, _ = self.get_mapping(vvar)
        n_fa, n_qp, dim, n_fn, n_c = self.get_data_shape(vvar)

        ebf = expand_basis(sg.bf, dim)

        return val_qp, ebf, mat, sg, diff_var



from __future__ import absolute_import
from __future__ import print_function
import numpy as nm
from .bspline import BSpline
from sfepy.base.base import Struct
from six.moves import range
from sfepy.base.compat import lstsq

class SplineBox(Struct):
    """
    B-spline geometry parametrization. The geometry can be modified
    by moving spline control points.
    """
    @staticmethod
    def gen_cp_idxs(ncp):
        dim = len(ncp)

        if dim == 2:
            idxs = nm.mgrid[0:ncp[0], 0:ncp[1]]
        elif dim == 3:
            idxs = nm.mgrid[0:ncp[0], 0:ncp[1], 0:ncp[2]]
        else:
            raise(ValueError)

        cp_idxs = []
        mul_idx = [1]
        for ii in range(dim - 1):
            mul_idx.append(mul_idx[ii] * ncp[ii])

        cp_idxs = []
        for ii in range(dim):
            cp_idxs.append(idxs[ii].reshape(nm.prod(ncp), order='F'))

        return cp_idxs, nm.array(mul_idx)

    @staticmethod
    def create_spb(bbox, coors, degree=3, nsg=None):
        nc, cdim = coors.shape
        inside = nm.ones((nc,), dtype=nm.bool)
        nsg = nm.ones((cdim,), dtype=nm.int) if nsg is None else nm.array(nsg)

        for idim in range(cdim):
            inrange = nm.logical_and(coors[:, idim] >= bbox[idim][0],
                                     coors[:, idim] <= bbox[idim][1])
            inside = nm.logical_and(inside, inrange)

        ncp_tot = 1
        base, uidx, ncp, cp = [], [], [], []
        for idim in range(cdim):
            ucoors, ucoors_idx = nm.unique(coors[inside, idim],
                                           return_inverse=True)
            ncp0 = degree + nsg[idim]
            bspl = BSpline(degree, ncp=ncp0)
            bspl.make_knot_vector(knot_range=(bbox[idim][0], bbox[idim][1]))
            knots = bspl.get_knot_vector()
            cp0 = nm.zeros((ncp0,), dtype=nm.double)
            for j in range(cp0.shape[0]):
                cp0[j] = nm.sum(knots[(j + 1):(j + degree + 1)]) / degree

            base.append(bspl.eval_basis(t=ucoors, return_val=True))
            uidx.append(ucoors_idx)
            ncp.append(ncp0)
            cp.append(cp0)
            ncp_tot *= ncp0

        cp_coors = nm.zeros((ncp_tot, cdim), dtype=nm.double)
        cp_idx, mul_cp_idx = SplineBox.gen_cp_idxs(ncp)
        for ii in range(cdim):
            cp_coors[:, ii] = cp[ii][cp_idx[ii]]

        return {'base': base,
                'uidx': uidx,
                'ncp': ncp,
                'cp_idx': cp_idx,
                'mul_cp_idx': mul_cp_idx,
                'cp_coors': cp_coors,
                'idxs_inside': inside}

    def __init__(self, bbox, coors, nsg=None, field=None):
        """
        Create a SplineBox.

        Parameters
        ----------
        bbox : array
            The mesh bounding box.
        coors : array
            The coordinates of mesh nodes.
        nsg : array
            The number of segments.
        """
        bbox = nm.asarray(bbox)
        coors = nm.asarray(coors)

        self.coors = coors.copy()
        self.cdim = coors.shape[1]

        if field is not None:
            field = nm.asarray(field)
            if len(field.shape) <= 1:
                field = field[..., nm.newaxis]
            self.field = field.copy()
        else:
            self.field = self.coors

        self.__dict__.update(self.create_spb(bbox, coors, nsg=nsg))

        if field is not None:
            if hasattr(self, 'idxs_inside'):
                b = field[self.idxs_inside, ...]
            else:
                b = field

            a = self.get_box_matrix()
            self.cp_values = lstsq(a, b)[0]

        else:
            self.cp_values = self.cp_coors

        self.cp_values0 = self.cp_values.copy()

    def get_coors_shape(self):
        """
        Get the shape of the coordinates.
        """
        return self.coors.shape

    def get_control_points(self, init=False):
        """
        Get the spline control points coordinates.

        Returns
        -------
        cpt_coors : array
            The coordinates of the spline control points.
        init : bool
            If True, return the initial state.
        """
        if init:
            return self.cp_values0

        else:
            return self.cp_values

    def set_control_points(self, cpt_coors, add=False):
        """
        Set the spline control points position.

        Parameters
        ----------
        cpt_coors : array
            The coordinates of the spline control points.
        add : bool
            If True, coors += cpt_coors
        """
        if add:
            self.cp_values += cpt_coors
        else:
            self.cp_values = cpt_coors

    def move_control_point(self, cpoint, val):
        """
        Change shape of spline parametrization.

        Parameters
        ----------
        cpoint : int, list
            The position (index or grid indicies) of the spline control point.
        val : array
            Displacement.
        """
        if type(cpoint) in [list, tuple, nm.ndarray]:
            idx = nm.dot(nm.array(cpoint), self.mul_cp_idx)
        else:
            idx = cpoint
        self.cp_values[idx, :] += val

    def get_box_matrix(self):
        """
        Returns:
            mtx : 2D array
                The matrix containing the coefficients of b-spline
                basis functions.
        """
        ncp, cdim = self.cp_coors.shape
        mtx = nm.ones((self.uidx[0].shape[0], ncp), dtype=nm.double)
        for ii in range(cdim):
            mtx *= self.base[ii][self.uidx[ii], :][:, self.cp_idx[ii]]

        return mtx

    def evaluate(self, cp_values=None, outside=True):
        """
        Evaluate the new position of the mesh coordinates.

        Parameters
        ----------
        cp_values : array
            The actual control point values. If None, use self.control_values.
        outside : bool
            If True, return also the coordinates outside the spline box.

        Returns
        -------
        new_coors : array
            The new position of the mesh coordinates.
        """
        if cp_values is None:
            cp_values = self.cp_values

        mtx = self.get_box_matrix()

        if outside and hasattr(self, 'idxs_inside'):
            field = self.field.copy()
            field[self.idxs_inside, ...] = nm.dot(mtx, cp_values)
            return field
        else:
            return nm.dot(mtx, cp_values)

    def evaluate_derivative(self, cpoint, dirvec):
        """
        Evaluate derivative of the spline
        in a given control point and direction.

        Parameters
        ----------
        cpoint : int, list
            The position (index or grid indicies) of the spline control point.
        dirvec : array
            The directional vector.

        Returns
        -------
        diff : array
            The derivative field.
        """
        if type(cpoint) in [list, tuple, nm.ndarray]:
            idxs = cpoint
        else:
            idxs = []
            aux = cpoint
            for ii in range(self.cdim):
                idxs.append(aux // self.mul_cp_idx[-(ii + 1)])
                aux = aux % self.mul_cp_idx[-(ii + 1)]
            idxs = idxs[::-1]

        aux = nm.ones((self.uidx[0].shape[0],), dtype=nm.double)
        for ii in range(self.cdim):
            aux *= self.base[ii][self.uidx[ii], idxs[ii]]

        dirvec = nm.asarray(dirvec)
        return nm.dot(aux[:, nm.newaxis],
                      nm.reshape(dirvec, (1, self.cp_values.shape[1])))

    def write_control_net(self, filename, deform_by_values=True):
        """
        Write the SplineBox shape to the VTK file.

        Parameters
        ----------
        filename : str
            The VTK file name.
        """
        ncp = self.ncp
        npt = nm.prod(ncp)

        f = open(filename, 'w')
        f.write("# vtk DataFile Version 2.6\nspbox file\n"
                "ASCII\nDATASET UNSTRUCTURED_GRID\n\n")
        f.write("POINTS %d float\n" % npt)

        if self.cdim == 2:
            ptformat = "%e %e 0.0\n"

        elif self.cdim == 3:
            ptformat = "%e %e %e\n"

        cp_coors = self.cp_values if deform_by_values else self.cp_coors
        for cpt in cp_coors:
            f.write(ptformat % tuple(cpt))

        cells = nm.array([nm.arange(0, ncp[0] - 1), nm.arange(1, ncp[0])]).T
        cp = ncp[0]
        nc = cp - 1
        for ii in range(1, self.cdim):
            cells1 = []
            ncpi = ncp[ii]
            for jj in range(ncpi):
                cells1.append(cells + jj * cp)
            nc = nc * ncpi

            cells = nm.array([nm.arange(0, ncpi - 1),
                              nm.arange(1, ncpi)]).T * cp
            for jj in range(cp):
                cells1.append(cells + jj)
            nc += (ncpi - 1) * cp

            cells = nm.vstack(cells1)
            cp *= ncp[ii]

        f.write("\nCELLS %d %d\n" % (nc, 3 * nc))
        for ii in cells:
            f.write("2 %d %d\n" % tuple(ii))
        f.write("\nCELL_TYPES %d\n" % nc)
        f.write("3\n" * nc)

        f.write("\nPOINT_DATA %d\n" % npt)
        for ival in range(self.cp_values.shape[1]):
            f.write("\nSCALARS cp_value_%d float 1\n" % (ival + 1))
            f.write("LOOKUP_TABLE default\n")
            f.write(str(b'\n'.join(self.cp_values[:, ival].astype('|S10'))) + '\n')

        f.close()


class SplineRegion2D(SplineBox):
    """
    B-spline geometry parametrization. The boundary of the SplineRegion2D
    is defined by BSpline curves.
    """
    @staticmethod
    def points_in_poly(points, poly, tol=1e-6):
        """
        Find which points are located inside the polygon.
        """
        poly = nm.array(poly)
        points = nm.array(points)
        inside = nm.zeros((points.shape[0],), dtype=nm.bool)

        p1 = poly[:-1]
        p2 = poly[1:]
        a1 = (p2[:, 1] - p1[:, 1])
        a1nz = nm.where(nm.fabs(a1) > 1e-16)[0]
        a2 = (p2[a1nz, 0] - p1[a1nz, 0]) / a1[a1nz]
        for jj, pt in enumerate(points):
            # on edges?
            if nm.any(nm.linalg.norm(p1 - pt, axis=1) +
                      nm.linalg.norm(p2 - pt, axis=1) -
                      nm.linalg.norm(p1 - p2, axis=1) < tol):
                inside[jj] = True
                continue

            # inside?
            val = nm.logical_and(
                (p1[a1nz, 1] > pt[1]) != (p2[a1nz, 1] > pt[1]),
                pt[0] < (a2*(pt[1] - p1[a1nz, 1]) + p1[a1nz, 0]))

            if (nm.where(val)[0].shape[0] % 2) > 0:
                inside[jj] = True

        return nm.where(inside)[0]

    @staticmethod
    def define_control_points(cp_bnd_coors, ncp):
        """
        Find positions of "inner" control points depending on boundary splines.
        """
        nx, ny = ncp
        grid = nm.zeros(ncp, dtype=nm.int32)
        grid.T.flat = nm.arange(nx * ny)

        coors = nm.zeros((nx * ny, 2), dtype=nm.float64)
        idxs1 = nm.arange(nx)
        idxs2 = nm.arange(1, ny - 1) * nx
        bcnd = nm.hstack([idxs1, idxs2 + nx - 1,
                          idxs1[::-1] + nx * (ny - 1), idxs2[::-1]])
        coors[bcnd, :] = cp_bnd_coors

        for ii in range(1, nx - 1):
            for jj, t in enumerate(nm.linspace(0, 1, ny)[1:-1]):
                c = (1 - t) * coors[ii, :] + t * coors[ii + (ny - 1)*nx, :]
                coors[ii + nx*(jj + 1), :] = c

        inside = grid[1:-1, 1:-1].flatten()
        for iiter in range(5):
            for ii in inside:
                dx = nm.array([0., 0.])
                for jj in [-1, +1, -nx, + nx]:
                    dx -= 0.25 * (coors[ii, :] - coors[ii + jj, :])
                coors[ii] += 0.1 * dx

        return coors

    @staticmethod
    def create_spb(spl_bnd, coors, rho=10):
        """
        Initialize SplineBox knots, control points, base functions, ...
        """
        dim = 2
        if coors.shape[1] != dim:
            print('Only 2D SplineBoxSBnd is supported!')
            raise(ValueError)

        bnd_poly = []
        bnd_cp = []
        for s in spl_bnd:
            s.set_param_n(int(rho))
            bnd_poly.append(s.eval()[:-1])
            bnd_cp.append(s.get_control_points()[:-1, :])

        bnd_poly.append(bnd_poly[0][0, :])
        ncpoints = 1
        base, bspl, uidx, ncp = [], [], [], []
        for si in [0, 1]:
            s = spl_bnd[si]
            bspl0 = BSpline(s.degree, ncp=s.ncp)
            bspl0.set_knot_vector(s.knots)
            bspl.append(bspl0)
            base.append(None)
            uidx.append(None)
            ncp.append(s.ncp)
            ncpoints *= s.ncp

        cp_idx, mul_cp_idx = SplineBox.gen_cp_idxs(ncp)
        cp_coors = SplineRegion2D.define_control_points(nm.vstack(bnd_cp), ncp)
        idxs_inside = SplineRegion2D.points_in_poly(coors, nm.vstack(bnd_poly))

        return {'base': base,
                'bspl': bspl,
                'uidx': uidx,
                'ncp': ncp,
                'cp_idx': cp_idx,
                'mul_cp_idx': mul_cp_idx,
                'cp_coors': cp_coors,
                'idxs_inside': idxs_inside}

    def find_ts(self, coors):
        """
        Function finds parameters (t, s) corresponding to given points (coors).
        """
        from scipy.optimize import minimize

        def ptdist(x, coors, spb):
            for ii in range(spb.cdim):
                spb.base[ii] = spb.bspl[ii].eval_basis(t=x[ii],
                                                       return_val=True)

            coors_approx = spb.evaluate(outside=False)
            return nm.linalg.norm(coors - coors_approx)

        def gen_grid(spb, rho):
            grid = nm.mgrid[0:rho, 0:rho]
            t = nm.linspace(0, 1, rho)
            for ii in range(spb.cdim):
                spb.uidx[ii] = grid[ii, :].reshape(rho**self.cdim, order='F')
                spb.base[ii] = spb.bspl[ii].eval_basis(t=t, return_val=True)

            return spb.evaluate(outside=False)

        rho = 100
        grid = gen_grid(self, rho)

        for ii in range(self.cdim):
            self.uidx[ii] = nm.array([0])

        ts = nm.zeros((coors.shape[0], self.cdim), dtype=nm.float64)
        for ii, ic in enumerate(coors):
            idx = nm.argmin(nm.linalg.norm(grid - ic, axis=1))
            x0 = nm.array([idx % rho, idx // rho]) / (rho - 1.)
            ts[ii] = minimize(lambda x: ptdist(x, ic, self), x0,
                              method='nelder-mead',
                              options={'xtol': 1e-5, 'disp': False}).x

        return ts

    def __init__(self, spl_bnd, coors, rho=1e3):
        """
        Create a SplineBox which boundary is defined by B-spline curves.

        Parameters
        ----------
        spl_bnd : list
            The list of BSpline objects (counterclockwise)
            defining the SplineBox boundary.
        coors : array
            The coordinates of the mesh nodes.
        rho : float
            The density of points defining the boundary polygon.
        """
        coors = nm.asarray(coors)
        self.__dict__.update(self.create_spb(spl_bnd, coors, rho))
        self.cdim = coors.shape[1]
        self.coors = coors.copy()
        self.field = self.coors
        self.cp_values = self.cp_coors

        self.ts = self.find_ts(coors[self.idxs_inside, :])

        for idim in range(self.cdim):
            ucoors, ucoors_idx = nm.unique(self.ts[:, idim],
                                           return_inverse=True)
            self.base[idim] = self.bspl[idim].eval_basis(t=ucoors,
                                                         return_val=True)
            self.uidx[idim] = ucoors_idx
